-- Function: sprepoctisklad(integer)
-- DROP FUNCTION sprepoctisklad(integer);
CREATE OR REPLACE FUNCTION sprepoctisklad(i_ucetobd integer)
  RETURNS void AS
$BODY$     
/* Prepocte stavy na skladove karte s id = iSkladKartaId */     
declare     
  /* Deklarace promennych */     
  _zaznam Record; 
  _skladKarta Record; 
  _dPolSkladPomoc Record;
  _vyrobniCislo Record;
  _pocatek_ucetobd Date; 
  _konec_ucetobd Date; 
  _typPohybuk character varying(255); 
  _sklfifo boolean;
  _evidVyrCis boolean;
  _prumCenaTuz numeric(19,6); 
  tmp_polsklad_exists boolean; 
   
begin 
  -- vytvoreni temporary tabulky na vypnuti triggeru 
  tmp_polsklad_exists := (select istableexist('tmp_polsklad')); 
  if(tmp_polsklad_exists <> true) then 
    CREATE TEMPORARY TABLE tmp_polsklad ( 
      idpolsklad integer 
    ) WITH (OIDS=TRUE); 
  end if; 
 
  -- nacteni datumu zacatku ucetniho obdobi 
  select platioddata, platidodata into _pocatek_ucetobd, _konec_ucetobd from nucetobdobi where iducetobdobi = i_ucetobd; 
 
  -- smazani tabulky fifo vazeb 
  delete from sfifovazby where idpolskladvydej in  
    (select idpolsklad from dpolsklad where  
            datVyst >= _pocatek_ucetobd and datVyst <= _konec_ucetobd); 
 
  -- nastaveni skladove karty na den zacatku ucetniho obdobi 
  for _skladKarta in select * from skarty where iducetobdobi = i_ucetobd loop 
    _prumCenaTuz := 0; 
    if(_skladKarta.pocatMj <> 0.0) then 
      _prumCenaTuz := _skladKarta.pocatTuz / _skladKarta.pocatMj; 
    end if; 
    update skarty set  
      prumcenatuz = _prumCenaTuz,  
      stavmj = _skladKarta.pocatMj,  
      stavtuz = _skladKarta.pocatTuz, 
      stavmen = _skladKarta.pocatMen 
      where idkarty = _skladKarta.idKarty; 
  end loop; 
 
  -- vypnuti triggeru 
  --alter table dpolsklad disable trigger all; 
  --alter table dpolsklad enable trigger dpolskladupdatezauctovano; 
  alter table dpolsklad disable trigger dpolskladinsert; 
  alter table dpolsklad disable trigger dpolskladupdate; 
  alter table dpolsklad disable trigger dpolskladdelete; 
 
  -- prepocet skladu 
  for _zaznam in select * from dpolsklad  
    where datVyst >= _pocatek_ucetobd and datVyst <= _konec_ucetobd 
    order by DatVyst, idPolSklad asc loop 
    select typpohybuk into _typPohybuk from ddoklsklad where iddoklsklad = _zaznam.iddoklsklad;  
    --pokud výdej, tak fifuju vazby  
    if (_typPohybuk = 'typPohybu.vydej') then
      select evidVyrCis into _evidVyrCis from ccenik where idcenik = _zaznam.idCenik;
      if (not _evidVyrCis) then
        execute sfifoVydej(_zaznam.idPolSklad, _zaznam.idCenik, _zaznam.idSkladu, _zaznam.idKarty, _zaznam.mnozmj);
      else
        for _vyrobniCislo in select * from svyrobnicislo where idpolskladvydej = _zaznam.idPolSklad loop
          update svyrobnicislo set idpolskladvydej = null where idvyrobnicislo = _vyrobniCislo.idvyrobnicislo;
          update svyrobnicislo set idpolskladvydej = _zaznam.idPolSklad where idvyrobnicislo = _vyrobniCislo.idvyrobnicislo;
        end loop;
      end if;
      execute sPrepoctiSumyDokladu(_zaznam.idDoklSklad); 
    end if;
 
    -- prumerne ceny
    _sklfifo := (select isfifosklad(_zaznam.datvyst)); 
    if (_sklfifo <> true) then   
      execute sZmenCenyNaPolSklad(_zaznam.idPolSklad, _zaznam.idkarty, _zaznam.iddoklsklad);     
    end if; 
 
    select * into _dPolSkladPomoc from dpolsklad where idpolsklad = _zaznam.idpolsklad; 
     
    execute sPrepoctiSkladKartu(_dPolSkladPomoc.MnozMJ,_dPolSkladPomoc.CenaMJ,_dPolSkladPomoc.DatVyst,_dPolSkladPomoc.idKarty,_dPolSkladPomoc.idDoklSklad,_dPolSkladPomoc.idskladu,1); 
  end loop;  
 
  -- zapnuti triggeru 
  alter table dpolsklad enable trigger dpolskladinsert; 
  alter table dpolsklad enable trigger dpolskladupdate; 
  alter table dpolsklad enable trigger dpolskladdelete; 
end;     
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION sprepoctisklad(integer) OWNER TO %firmaRole%;

update svyrobnicislo set idpolskladvydej = null where idpolskladvydej not in (select idpolsklad from dpolsklad);
update svyrobnicislo set idpolfakvydej = null where idpolfakvydej not in (select idpolfak from dpolfak);
update svyrobnicislo set idpolintvydej = null where idpolintvydej not in (select idpolint from dpolint);
update svyrobnicislo set idpolskladprijem = null where idpolskladprijem not in (select idpolsklad from dpolsklad);
update svyrobnicislo set idpolfakprijem = null where idpolfakprijem not in (select idpolfak from dpolfak);
update svyrobnicislo set idpolintprijem = null where idpolintprijem not in (select idpolint from dpolint);

-- chybejici constrainty pro vyrobni cisla
ALTER TABLE svyrobnicislo
  ADD CONSTRAINT idpolskladprijem_fk FOREIGN KEY (idpolskladprijem)
      REFERENCES dpolsklad (idpolsklad) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

ALTER TABLE svyrobnicislo
  ADD CONSTRAINT idpolskladvydej_fk FOREIGN KEY (idpolskladvydej)
      REFERENCES dpolsklad (idpolsklad) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

ALTER TABLE svyrobnicislo
  ADD CONSTRAINT idpolfakprijem_fk FOREIGN KEY (idpolfakprijem)
      REFERENCES dpolfak (idpolfak) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

ALTER TABLE svyrobnicislo
  ADD CONSTRAINT idpolfakvydej_fk FOREIGN KEY (idpolfakvydej)
      REFERENCES dpolfak (idpolfak) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

ALTER TABLE svyrobnicislo
  ADD CONSTRAINT idpolintprijem_fk FOREIGN KEY (idpolintprijem)
      REFERENCES dpolint (idpolint) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

ALTER TABLE svyrobnicislo
  ADD CONSTRAINT idpolintvydej_fk FOREIGN KEY (idpolintvydej)
      REFERENCES dpolint (idpolint) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;


insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 102, 't');
