-------------------------------------------------------------------------------------
-- Bug 13858 – [WinStrom] Problém při vytvoření vydané faktury
-------------------------------------------------------------------------------------

create or replace function dGetCisDokl
(in i_idDokl integer, in i_idRady integer, in i_idUcetniObdobi integer, in i_Modul varchar(3), in i_TypPohybuK varchar(50))
returns refcursor as $$
declare
  _idRadyRok integer;
  _CisDelka integer;
  _ZobrazNuly boolean;
  _CisAkt integer;
  _CisPoc integer;
  _Prefix varchar(8);
  _Postfix varchar(8);
  _PocetZaznamu integer;

  _idRadyDiry integer;
  _CisDira varchar(20);
  _Unique boolean;
  _iter integer;

  o_Chyba integer;
  o_CisDokl varchar(20);

  _navratoveHodnoty refcursor;
begin
  o_Chyba:= 0;
  o_CisDokl:= null;

  /* Ověření, zda pro daný účetní rok existuje dokladová řada pro rok. Pokud neexistuje, vytvoří se. */
  select idRadyRok into _idRadyRok from dRadyRok where idRady = i_idRady and iducetobdobi = i_idUcetniObdobi;
  if(_idRadyRok is null) then
    execute dZalozRaduRok (i_idRady, i_idUcetniObdobi, i_Modul);
  end if;

  /* Načte se dokladová řada pro účetní rok */
  select CisDelka, ZobrazNuly, CisAkt, CisPoc, Prefix, Postfix, idRadyRok into _CisDelka,_ZobrazNuly,_CisAkt,_CisPoc,_Prefix,_Postfix,
    _idRadyRok from dRadyRok where idRady = i_idRady and iducetobdobi = i_idUcetniObdobi for update;


  /* Podíváme se do seznamu děr, zda tam náhodou neexistuje vrácené číslo. Pokud ano, zjistíme jeho unikátnost. Pokud je unikátní,
     provedeme jeho přidělení klientu. V opačném případě jej smažeme a pokusíme se vybrat nové číslo ze seznamu děr. */
  select CisDira, idRadyDiry into _CisDira,_idRadyDiry from dRadyDiry where idRadyRok = _idRadyRok and LastUpdate <= now() order by cisdira asc limit 1;
  if (_CisDira is not null) then

    _Unique:= (select dUniqueCisDokl(-1,i_Modul,i_TypPohybuK,_CisDira));
    while(_Unique = false and _CisDira is not null) loop
      delete from dRadyDiry where idRadyDiry = _idRadyDiry;

      select CisDira, idRadyDiry into _CisDira,_idRadyDiry from dRadyDiry where idRadyRok = _idRadyRok and LastUpdate <= now() order by CisDira asc limit 1;
      if(_CisDira is not null) then
        _Unique:= (select dUniqueCisDokl(-1,i_Modul,i_TypPohybuK,_CisDira));
      end if;

    end loop;

    /* Pokud bylo nalezeno unikátní číslo, provede se jeho rezervace a vrátí se klientu. */
    if(_Unique = true and _CisDira is not null) then
      o_CisDokl:= _CisDira;
      update dRadyDiry set LastUpdate = now() + interval '2 hours' where idRadyDiry = _idRadyDiry;
    end if;
  end if;

  /* Pokud nebylo číslo vzato z děr, provede se jeho přidělení standardním způsobem z řady.
     Pozor! Pokud se jedná o první číslo řady, je potřeba vrátit aktuální číslo a nikoliv inkrementované. */
  if (o_CisDokl is null) then
    if (_CisAkt <> _CisPoc) then
      _CisAkt:= _CisAkt+1;
    end if;

    /* Získá se číslo a provede se test na jeho unikátnost.*/
    o_CisDokl:= (select dVytvorCisDokl(_Prefix,_CisAkt,_Postfix,_ZobrazNuly,_CisDelka,i_idUcetniObdobi));
    _Unique:= (select dUniqueCisDokl(-1,i_Modul,i_TypPohybuK,o_CisDokl));

    /* Podíváme se také do děr, protože počátek řady mohl být již vyžádán a je umístěn v dírách.
       Typickým příkladem je generování závazků u Leasingu, kde se vytahuje několik čísel najednou. */
    if (_Unique = true) then
      select count(*) into _PocetZaznamu from dRadyDiry where cisdira=o_CisDokl and idradyrok=_idRadyRok;
      if (_PocetZaznamu > 0) then
        _Unique:= false;
      end if;
    end if;

    /* Pokud není číslo unikátní, pokus se vygenerovat nové. */
    _iter:= 0;
    while (_Unique = false) loop
      _CisAkt:= _CisAkt+1;
      _iter:= _iter + 1;

      o_CisDokl:= (select dVytvorCisDokl(_Prefix,_CisAkt,_Postfix,_ZobrazNuly,_CisDelka,i_idUcetniObdobi));
      _Unique:=(select dUniqueCisDokl(-1,i_Modul,i_TypPohybuK,o_CisDokl));

      if (_iter > 100000) then
        /* Číslo se z neznámých důvodů nepodařilo vygenerovat. Pozn.: Ochrana proti zacyklení.*/
        raise exception 'dGetCisDokl(): Číslo dokladu se nepodařilo vygenerovat.';
      end if;
    end loop;

    /* Nastavení chybových stavů (pokud nějaké nastaly). */
    if (o_CisDokl = 'x') then /* Kapacita číselné řady vyčerpána. */
      o_Chyba:= 1;
    end if;
    if (o_CisDokl = 'xx') then /* Délka číselné řady je delší. Max. 20 znaků.  */
      o_Chyba:= 2;
    end if;
    if (o_CisDokl = 'xxx') then /* Maximální délka čísla smí být 12 znaků. */
      o_Chyba:= 3;
    end if;

    /* Podařilo se vygenerovat unikátní číslo, provede se jeho rezervace a nastaví se nový vrchol dokladové řady. */
    if (o_Chyba = 0) and (select count(*) from dRadyDiry where idRadyRok = _idRadyRok and CisDira = o_CisDokl) = 0 then
      insert into dRadyDiry(idRadyRok,CisDira,LastUpdate) values(_idRadyRok,o_CisDokl, now() + interval '2 hours');
      update dRadyRok set cisAkt = _cisAkt where idRadyRok = _idRadyRok;
    end if;

  end if;

  open _navratoveHodnoty for select o_CisDokl as CisloDokladu, o_Chyba as Cislo;
  return _navratoveHodnoty;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dGetCisDokl(in i_idDokl integer, in i_idRady integer, in i_idUcetniObdobi integer, in i_Modul varchar(3), in i_TypPohybuK varchar(50)) OWNER TO %firmaRole%;


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 139, 't');
