-------------------------------------------------------
-- Bug 15818 – [WinStrom] Záložka "Ceny pro odběratele/speciální kód"
-------------------------------------------------------

CREATE SEQUENCE seq_codberatele
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_codberatele OWNER TO %firmaRole%;

CREATE TABLE codberatele (
    idodberatel integer DEFAULT nextval('seq_codberatele'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,

    kodindi character varying(20),
    prodejcena numeric(19,6) DEFAULT 0.0 NOT NULL,
    poznam text,

    idcenik integer NOT NULL,
    idfirmy integer NOT NULL,

    CONSTRAINT codberatele_pkey PRIMARY KEY (idodberatel),
    CONSTRAINT codberatele_idcenik FOREIGN KEY (idcenik)
        REFERENCES ccenik (idcenik)
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT codberatele_idfirmy FOREIGN KEY (idfirmy)
        REFERENCES aadresar (idfirmy)
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT codberatele_unique_key UNIQUE (idcenik, idfirmy)
);

ALTER TABLE public.codberatele OWNER TO %firmaRole%;

CREATE INDEX codberatele_idcenik_index ON codberatele USING btree (idcenik);
CREATE INDEX codberatele_idfirmy_index ON codberatele USING btree (idfirmy);


-------------------------------------------------------
-- Bug 15819 – [WinStrom] Vazby-sady, komplety
-------------------------------------------------------

CREATE SEQUENCE seq_csady
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_csady OWNER TO %firmaRole%;

CREATE TABLE csady (
    idsady integer DEFAULT nextval('seq_csady'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,

    mnozmj numeric(19,6) DEFAULT 0.0 NOT NULL,
    poznam text,

    idceniksada integer NOT NULL,
    idcenik integer NOT NULL,

    CONSTRAINT csady_pkey PRIMARY KEY (idsady),
    CONSTRAINT csady_idceniksada FOREIGN KEY (idceniksada)
        REFERENCES ccenik (idcenik)
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT csady_idcenik FOREIGN KEY (idcenik)
        REFERENCES ccenik (idcenik)
        ON UPDATE NO ACTION ON DELETE NO ACTION
);

ALTER TABLE public.csady OWNER TO %firmaRole%;

CREATE INDEX csady_idceniksada_index ON csady USING btree (idceniksada);
CREATE INDEX csady_idcenik_index ON csady USING btree (idcenik);


-------------------------------------------------------
-- Bug 15820 – [WinStrom] Vazby-Poplatky
-------------------------------------------------------

CREATE SEQUENCE seq_cpoplatky
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_cpoplatky OWNER TO %firmaRole%;

CREATE TABLE cpoplatky (
    idpoplatek integer DEFAULT nextval('seq_cpoplatky'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,

    mnozMj numeric(19,6) DEFAULT 0.0 NOT NULL,
    poznam text,

    idcenikotec integer NOT NULL,
    idcenik integer NOT NULL,

    CONSTRAINT cpoplatky_pkey PRIMARY KEY (idpoplatek),
    CONSTRAINT cpoplatky_idcenikotec FOREIGN KEY (idcenikotec)
        REFERENCES ccenik (idcenik)
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT cpoplatky_idcenik FOREIGN KEY (idcenik)
        REFERENCES ccenik (idcenik)
        ON UPDATE NO ACTION ON DELETE NO ACTION
);

ALTER TABLE public.cpoplatky OWNER TO %firmaRole%;

CREATE INDEX cpoplatky_idcenikotec_index ON cpoplatky USING btree (idcenikotec);
CREATE INDEX cpoplatky_idcenik_index ON cpoplatky USING btree (idcenik);


-------------------------------------------------------
-- Bug 15821 – [WinStrom] Vazby-Podobné
-------------------------------------------------------

CREATE SEQUENCE seq_cpodobne
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_cpodobne OWNER TO %firmaRole%;

CREATE TABLE cpodobne (
    idpodobne integer DEFAULT nextval('seq_cpodobne'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,

    poznam text,

    idcenikotec integer NOT NULL,
    idcenik integer NOT NULL,

    CONSTRAINT cpodobne_pkey PRIMARY KEY (idpodobne),
    CONSTRAINT cpodobne_idcenikotec FOREIGN KEY (idcenikotec)
        REFERENCES ccenik (idcenik)
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT cpodobne_idcenik FOREIGN KEY (idcenik)
        REFERENCES ccenik (idcenik)
        ON UPDATE NO ACTION ON DELETE NO ACTION
);

ALTER TABLE public.cpodobne OWNER TO %firmaRole%;

CREATE INDEX cpodobne_idcenikotec_index ON cpodobne USING btree (idcenikotec);
CREATE INDEX cpodobne_idcenik_index ON cpodobne USING btree (idcenik);


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 156, 't');
