-- Bug 10909 – [WinStrom] Při změně zaúčtování protiúčtu skrze trigger se neměni flag zauctovano
-- realizace triggeru udrzujici priznak zuctovano

-- funkce pro zjisteni zda je doklad zauctovan, idDokl - id doklad, doklPostfix - konec nazvu tabulky dokladu/polozek (Fak/Int/Sklad)
CREATE OR REPLACE FUNCTION isDokladZuctovan(idDokl integer, doklPostfix varchar(100))
  RETURNS boolean AS $$
declare
  _ucetni boolean;
  _bezpol boolean; 
  _doklOk integer;
  _polozek integer;
  _polOk integer;
  idDoklColumn varchar(100);
  doklTable varchar(100);
  polTable varchar(100);
begin
  idDoklColumn := 'idDokl' || doklPostfix;
  doklTable := 'dDokl' || doklPostfix;
  polTable := 'dPol' || doklPostfix;

  execute 'select ucetni, bezpolozek from ' || doklTable || ' where ' || idDoklColumn || ' = ' || idDokl into _ucetni, _bezpol;
  execute 'select count(*) from ' || polTable || ' where ' || idDoklColumn || ' = ' || idDokl into _polozek;
  _doklOk := 0;

  if (_ucetni = true) then
    if (_bezpol = true or _polozek = 0) then
      execute
        'select count(*) from ' || doklTable || ' where ' || idDoklColumn || ' = ' || idDokl || ' and idstred is not null and
          datucto is not null and idprimucet is not null and idprotiucet is not null and
          (sumdphsniz = 0 or iddphsnizucet is not null) and (sumdphzakl = 0 or iddphzaklucet is not null)'
        into _doklOk;
    else
      execute
        'select count(*) from ' || polTable || ' where ' || idDoklColumn || ' = ' || idDokl || ' and idstred is not null and
          datucto is not null and idzkldalucet is not null and idzklmducet is not null and
          (modul = ''SKL'' or typszbdphk is null or typszbdphk = ''typSzbDph.dphOsv'' or (
              iddphmducet is not null and iddphdalucet is not null))'
        into _polOk;
      if (_polOk = _polozek) then
        _doklOk := 1;
      end if;
    end if;
  end if;
  
  return (_doklOk = 1);
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.isDokladZuctovan(integer, varchar) OWNER TO %firmaRole%;


-- trigger na fakturach
CREATE OR REPLACE FUNCTION ddoklfakupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklFak, 'Fak') into _zuctovano;
  
  if (NEW.zuctovano <> _zuctovano) then
    NEW.zuctovano = _zuctovano;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION ddoklfakupdatezauctovano() OWNER TO %firmaRole%;

CREATE TRIGGER ddoklfakupdatezauctovano AFTER UPDATE ON ddoklfak
  FOR EACH ROW EXECUTE PROCEDURE ddoklfakupdatezauctovano();


-- trigger na bance/pokladne/internich dokladech
CREATE OR REPLACE FUNCTION ddoklintupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklInt, 'Int') into _zuctovano;
  
  if (NEW.zuctovano <> _zuctovano) then
    NEW.zuctovano = _zuctovano;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION ddoklintupdatezauctovano() OWNER TO %firmaRole%;

CREATE TRIGGER ddoklintupdatezauctovano AFTER UPDATE ON ddoklint
  FOR EACH ROW EXECUTE PROCEDURE ddoklintupdatezauctovano();


-- trigger na skladu
CREATE OR REPLACE FUNCTION ddoklskladupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklSklad, 'Sklad') into _zuctovano;
  
  if (NEW.zuctovano <> _zuctovano) then
    NEW.zuctovano = _zuctovano;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION ddoklskladupdatezauctovano() OWNER TO %firmaRole%;

CREATE TRIGGER ddoklskladupdatezauctovano AFTER UPDATE ON ddoklsklad
  FOR EACH ROW EXECUTE PROCEDURE ddoklskladupdatezauctovano();


-- trigger na polozkach faktur
CREATE OR REPLACE FUNCTION dpolfakupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklFak, 'Fak') into _zuctovano;
  
  update ddoklfak set zuctovano = _zuctovano where idDoklFak = NEW.idDoklFak and zuctovano <> _zuctovano;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION dpolfakupdatezauctovano() OWNER TO %firmaRole%;

CREATE TRIGGER dpolfakupdatezauctovano AFTER UPDATE ON dpolfak
  FOR EACH ROW EXECUTE PROCEDURE dpolfakupdatezauctovano();


-- trigger na polozkach banky/pokladny/internich dokladu
CREATE OR REPLACE FUNCTION dpolintupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklInt, 'Int') into _zuctovano;
  
  update ddoklint set zuctovano = _zuctovano where idDoklInt = NEW.idDoklInt and zuctovano <> _zuctovano;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION dpolintupdatezauctovano() OWNER TO %firmaRole%;

CREATE TRIGGER dpolintupdatezauctovano AFTER UPDATE ON dpolint
  FOR EACH ROW EXECUTE PROCEDURE dpolintupdatezauctovano();


-- trigger na polozkach skladu
CREATE OR REPLACE FUNCTION dpolskladupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklSklad, 'Sklad') into _zuctovano;
  
  update ddoklsklad set zuctovano = _zuctovano where idDoklSklad = NEW.idDoklSklad and zuctovano <> _zuctovano;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION dpolskladupdatezauctovano() OWNER TO %firmaRole%;

CREATE TRIGGER dpolskladupdatezauctovano AFTER UPDATE ON dpolsklad
  FOR EACH ROW EXECUTE PROCEDURE dpolskladupdatezauctovano();


-- zvyseni cisla verze db
insert into dbversion(iddbversion, dbversion, needreimport)
        values (nextval('seq_dbversion'), 20, 't');