--------------------------------------------------------------------------------
-- Bug 18563 – [WinStrom] Pridat tabulku mapovani skladu
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION errorInInsertKusovnik(_iddoklsklad integer, _datvyst date, _idcenik integer, _idskladu integer, _prevodka boolean)
  RETURNS character varying AS
$BODY$
declare
  _typPohybuK character varying;
  _typZasobyK character varying;
  _idKusovnik integer;
  potKus record;
  _cenik record;
  _sklKarta record;
  _iducetobdobi integer;
  _idSkladPotKus integer;
begin
  -- napriklad pokud se jedna o prevodku mezi sklady nebudeme z prijemky vytvaret prevodku
  if (_prevodka = true) then
    return null;
  end if;

  -- typem pohybu musi byt prijem
  select typpohybuk into _typPohybuK from ddoklsklad d where d.iddoklsklad = _iddoklsklad;
  if (_typPohybuK <> 'typPohybu.prijem') then
    return null;
  end if;

  -- idcenik odkazuje na neco co je soucasti kusovniku.
  select ckus.idkusovnik into _idKusovnik from ckusovnik ckus where ckus.idcenik = _idcenik
    and exists (select 1 from ckusovnik cPot where cPot.idotec = ckus.idkusovnik);
  if (_idKusovnik is null) then
    return null;
  end if;

  -- pro vsechny kusovniky, ktere jsou pod nasim kusovnikem a jsou skladove nebo vyroba
  for potKus in (select * from ckusovnik cPot where cPot.idotec = _idKusovnik) loop
    select * into _cenik from cCenik where idCenik = potKus.idCenik;

    select typZasobyK into _typZasobyK from cCenik cen where cen.idCenik = potKus.idCenik;
    if (_typZasobyK <> 'typZasoby.skladem' and _typZasobyK <> 'typZasoby.vyrobek') then
        continue;
    end if;

    select findIdUcetniObdobiKDatu(_datvyst) into _iducetobdobi;
    -- vybereme skladovou kartu podle preferovaneho skladu
    select null into _sklKarta;
    _idSkladPotKus = null;

    select getPrefIdSkladu(
	(select idstred from ddoklsklad where iddoklsklad = _iddoklsklad),
	_cenik.idskupzboz, potKus.idcenik) into _idSkladPotKus;

    if (_idSkladPotKus is not null) then
	select * into _sklKarta from skarty where idcenik = potKus.idcenik and idtypdokl = _idSkladPotKus and iducetobdobi = _iducetobdobi;
    end if;

    -- sklad karta nebyla nalezena podle preferovaneho skladu, vezmeme sklad stejny jako z vytvarene polozky
    if (_sklKarta is null) then
	select * into _sklKarta from skarty where idcenik = potKus.idcenik and idtypdokl = _idskladu and iducetobdobi = _iducetobdobi;
    end if;

    -- skladovou kartu k tomuhle ceniku nelze nalezt
    if (_sklKarta is null) then
	return _cenik.kod;
    end if;
  end loop;

  return null;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION errorInInsertKusovnik(integer, date, integer, integer, boolean) OWNER TO %firmaRole%;


--------------------------------------------------------------------------------
-- Bug 18642 – [WinStrom] Tisk faktur se seřazenými položkami
--------------------------------------------------------------------------------
alter table dtypdokl add column razeniProTiskK character varying(50);


--------------------------------------------------------------------------------
-- Oprava - nesla obnovit firma ze zalohy
--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION putglotxvariable(i_kod character varying, i_hodnota character varying)
  RETURNS void AS $$
DECLARE
  tmpGloTxVarTableExists boolean;
  pom integer;
BEGIN
  tmpGloTxVarTableExists := (SELECT istableexist('tmp_gloTxVar'));

  IF (tmpGloTxVarTableExists <> true) THEN
    CREATE TEMP TABLE tmp_gloTxVar (
      kod character varying(50),
      hodnota character varying(50)
    ) WITH (OIDS=FALSE) ON COMMIT DROP;
  END IF;

  SELECT count(1) INTO pom FROM tmp_gloTxVar WHERE kod=i_kod;
  if (pom > 0) then
    UPDATE tmp_gloTxVar SET hodnota=i_hodnota WHERE kod=i_kod;
  ELSE
    INSERT INTO tmp_gloTxVar VALUES(i_kod, i_hodnota);
  END IF;

END;
$$ LANGUAGE plpgsql;

ALTER FUNCTION putglotxvariable(i_kod character varying, i_hodnota character varying) OWNER TO %firmaRole%;


--------------------------------------------------------------------------------
-- Bug 18580 – [WinStrom] Zrušit přenášení protiúčtu u rozpadu kusovníku
--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION ddoklskladupdateucty() RETURNS trigger
    AS $$
declare idFak int;
--declare idSklad int;
begin
  if (NEW.typPohybuK = 'typPohybu.prijem' and compareId(NEW.idProtiUcet, OLD.idProtiUcet) = false) then
    for idFak in (select idTabA from dVazebTab where typVazbyK = 'typVazbyDokl.hlavaSkl' and idTabB = NEW.idDoklSklad and
            compareId((select idProtiUcet from dDoklFak where idDoklFak = idTabA), NEW.idProtiUcet) = false)
    loop
        execute dNastavUcetDokladu(0, idFak, 1, NEW.idProtiUcet);
    end loop;

    /*for idSklad in (select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.hlavaSklKus' and idTabA = NEW.idDoklSklad and
            compareId((select idProtiUcet from dDoklSklad where idDoklSklad = idTabB), NEW.idProtiUcet) = false)
    loop
        execute dNastavUcetDokladu(2, idSklad, 1, NEW.idProtiUcet);
    end loop;*/

  else
    /*if (NEW.typPohybuK = 'typPohybu.vydej' and compareId(NEW.idProtiUcet, OLD.idProtiUcet) = false) then

      for idSklad in (select idTabA from dVazebTab where typVazbyK = 'typVazbyDokl.hlavaSklKus' and idTabB = NEW.idDoklSklad and
              compareId((select idProtiUcet from dDoklSklad where idDoklSklad = idTabA), NEW.idProtiUcet) = false)
      loop
          execute dNastavUcetDokladu(2, idSklad, 1, NEW.idProtiUcet);
      end loop;

    end if;*/
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.ddoklskladupdateucty() OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION dpolskladupdateucty() RETURNS trigger
    AS $$
declare
  nelzeZmena int;
  _modul character varying;
begin
  if ((select typPohybuK from dDoklSklad where idDoklSklad = NEW.idDoklSklad) = 'typPohybu.prijem') then
    if (compareId(NEW.idZklDalUcet, OLD.idZklDalUcet) = false and NEW.kopZklDalUcet = false) then
      select modulA into _modul from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSkl';
      if(_modul like 'POK') then
        select count(*) into nelzeZmena from dPolInt where compareId(idZklMdUcet, NEW.idZklDalUcet) = false and kopZklMdUcet = true and
          idPolInt in (select idTabA from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSkl');
        --raise exception 'nelze zmena je % ....', nelzeZmena;
        if (nelzeZmena = 0) then
          update dPolInt set idZklMdUcet = NEW.idZklDalUcet where compareId(idZklMdUcet, NEW.idZklDalUcet) = false and
            idPolInt in (select idTabA from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSkl');
        else
          raise exception 'dPolSkladUpdateUcty(): Nelze zmenit ucet u navazane polozky.';
        end if;
      else
        select count(*) into nelzeZmena from dPolFak where compareId(idZklMdUcet, NEW.idZklDalUcet) = false and kopZklMdUcet = true and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSkl');
        --raise exception 'nelze zmena je % ....', nelzeZmena;
        if (nelzeZmena = 0) then
          update dPolFak set idZklMdUcet = NEW.idZklDalUcet where compareId(idZklMdUcet, NEW.idZklDalUcet) = false and
            idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSkl');
        else
          raise exception 'dPolSkladUpdateUcty(): Nelze zmenit ucet u navazane polozky.';
        end if;

        /*select count(*) into nelzeZmena from dPolSklad where compareId(idZklMdUcet, NEW.idZklDalUcet) = false and kopZklMdUcet = true and
            idPolSklad in (select idTabB from dVazebTab where idTabA = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSklKus');
        if (nelzeZmena = 0) then
          update dPolSklad set idZklMdUcet = NEW.idZklDalUcet where compareId(idZklMdUcet, NEW.idZklDalUcet) = false and
            idPolSklad in (select idTabB from dVazebTab where idTabA = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSklKus');
        else
          raise exception 'dPolSkladUpdateUcty(): Nelze zmenit ucet u navazane polozky. (typVazbyDokl.polozkaSklKus, prijem)';
        end if;*/
      end if;

    end if;
  else
    /*if (compareId(NEW.idZklMdUcet, OLD.idZklMdUcet) = false and NEW.kopZklMdUcet = false) then

      select count(*) into nelzeZmena from dPolSklad where compareId(idZklDalUcet, NEW.idZklMdUcet) = false and kopZklDalUcet = true and
          idPolSklad in (select idTabA from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSklKus');
      if (nelzeZmena = 0) then
        update dPolSklad set idZklDalUcet = NEW.idZklMdUcet where compareId(idZklDalUcet, NEW.idZklMdUcet) = false and
          idPolSklad in (select idTabA from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSklKus');
      else
        raise exception 'dPolSkladUpdateUcty(): Nelze zmenit ucet u navazane polozky. (typVazbyDokl.polozkaSklKus, vydej)';
      end if;

    end if;*/
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.dpolskladupdateucty() OWNER TO %firmaRole%;


--------------------------------------------------------------------------------
-- Bug 17557 – [WinStrom] Aktualizace kalkulačky na rok 2010
--------------------------------------------------------------------------------


create or replace function runupgrade()
    returns void as $$
declare
    stat character varying(10);
    pocet integer;
begin
  select a.uuid into stat from nnastav n  left join utyporg u on u.idtyporg = n.idtyporg left join astaty a on a.idstatu = u.idstatu
    where ((select max(platioddata) from nnastav) is null or (select max(platioddata) from nnastav) = platioddata);
  
  -- kalkulacku aktualizujeme jen pro cechy
  if (stat = 'CZ_CZ') then
-- zacatek ceskeho bloku
-- update slozek

-- update inicializacnich radek kalkulacky

-- update radek kalkulacky
UPDATE pmzdcalc SET koeficient = 0.135 WHERE idmzdcalc = 540;

-- konec ceskeho bloku
  end if; 
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION runupgrade() OWNER TO %firmaRole%;

select runupgrade();

drop function runupgrade();


--------------------------------------------------------------------------------


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 273, 't');
