-- Bug 11824 – [WinStrom] Účtují se i doklady, které nemají příznak zaúčtováno nastaven na true
-- návrat do skoro původního stavu, zůstává pouze oprava aby se neúčtovali položky, které nemají vše vyplněno
-- tato oprava je vyřešena ve funkci createOrUpdateObraty

-- zauctovani nezauctovanych polozek faktur
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
       P.sumZkl, P.sumZklMen) from dpolfak as P left outer join ddoklfak as D on D.idDoklFak = P.idDoklFak where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
       
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
       P.sumDph, P.sumDphMen) from dpolfak as P left outer join ddoklfak as D on D.idDoklFak = P.idDoklFak where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
       
       
-- zauctovani nezauctovanych polozek internich dokladu
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
       P.sumZkl, P.sumZklMen) from dpolint as P left outer join ddoklint as D on D.idDoklInt = P.idDoklInt where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
       
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
       P.sumDph, P.sumDphMen) from dpolint as P left outer join ddoklint as D on D.idDoklInt = P.idDoklInt where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
                                
                                
-- zauctovani nezauctovanych polozek skladovych dokladu                     
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
       P.sumZkl, P.sumZklMen) from dpolsklad as P left outer join ddoklsklad as D on D.idDoklSklad = P.idDoklSklad where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
       
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
       P.sumDph, P.sumDphMen) from dpolsklad as P left outer join ddoklsklad as D on D.idDoklSklad = P.idDoklSklad where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
                                
                                

-- navraceni triggeru ktery uctuje pri vytvoreni polozky do puvodni podoby
CREATE OR REPLACE FUNCTION dpoldoklinsertucto()
  RETURNS trigger AS $$
begin
  execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet,
        NEW.idMeny, NEW.idStred, NEW.sumZkl, NEW.sumZklMen);
  
  execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet,
        NEW.idMeny, NEW.idStred, NEW.sumDph, NEW.sumDphMen);
  
  return NEW;
end;
$$ LANGUAGE plpgsql;


-- navraceni triggeru ktery uctuje pri update polozky do puvodni podoby
CREATE OR REPLACE FUNCTION dpoldoklupdateucto()
  RETURNS trigger AS $$
begin
  
  execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet,
      OLD.idMeny, OLD.idStred, -OLD.sumZkl, -OLD.sumZklMen);
  
  execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet,
      OLD.idMeny, OLD.idStred, -OLD.sumDph, -OLD.sumDphMen);
  
  execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet,
      NEW.idMeny, NEW.idStred, NEW.sumZkl, NEW.sumZklMen);
  
  execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet,
      NEW.idMeny, NEW.idStred, NEW.sumDph, NEW.sumDphMen);
  
  return NEW;
end;
$$ LANGUAGE plpgsql;


-- navraceni triggeru ktery uctuje pri mazani polozky do puvodni podoby
CREATE OR REPLACE FUNCTION dpoldokldeleteucto()
  RETURNS trigger AS $$
begin
  execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet,
        OLD.idMeny, OLD.idStred, -OLD.sumZkl, -OLD.sumZklMen);
  
  execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet,
        OLD.idMeny, OLD.idStred, -OLD.sumDph, -OLD.sumDphMen);
  
  return OLD;
end;
$$ LANGUAGE plpgsql;


-- odstraneni nepotrebnych triggeru na dokladech
DROP TRIGGER dzdoklfakupdateucto ON ddoklfak;
DROP TRIGGER dzdoklintupdateucto ON ddoklint;
DROP TRIGGER dzdoklskladupdateucto ON ddoklsklad;
DROP FUNCTION ddoklupdateucto();


--------------------------------------------------------------------------------

-- uprava funkce pro zjisteni zda je doklad zauctovan, idDokl - id doklad, doklPostfix - konec nazvu tabulky dokladu/polozek (Fak/Int/Sklad)
-- pokud je doklad stornovan, zauctovano je vzdy false - jedna se pouze o kosmetickou zmenu
CREATE OR REPLACE FUNCTION isDokladZuctovan(idDokl integer, doklPostfix varchar(100))
  RETURNS boolean AS $$
declare
  _ucetni boolean;
  _storno boolean;
  _bezpol boolean; 
  _doklOk integer;
  _polozek integer;
  _polOk integer;
  idDoklColumn varchar(100);
  doklTable varchar(100);
  polTable varchar(100);
begin
  idDoklColumn := 'idDokl' || doklPostfix;
  doklTable := 'dDokl' || doklPostfix;
  polTable := 'dPol' || doklPostfix;

  execute 'select ucetni, bezpolozek, storno from ' || doklTable || ' where ' || idDoklColumn || ' = ' || idDokl into _ucetni, _bezpol, _storno;
  execute 'select count(*) from ' || polTable || ' where ' || idDoklColumn || ' = ' || idDokl into _polozek;
  _doklOk := 0;

  if (_ucetni = true and _storno = false) then
    if (_bezpol = true or _polozek = 0) then
      execute
        'select count(*) from ' || doklTable || ' where ' || idDoklColumn || ' = ' || idDokl || ' and idstred is not null and
          datucto is not null and idprimucet is not null and idprotiucet is not null and
          (sumdphsniz = 0 or iddphsnizucet is not null) and (sumdphzakl = 0 or iddphzaklucet is not null)'
        into _doklOk;
    else
      execute
        'select count(*) from ' || polTable || ' where ' || idDoklColumn || ' = ' || idDokl || ' and idstred is not null and
          datucto is not null and idzkldalucet is not null and idzklmducet is not null and
          (modul = ''SKL'' or typszbdphk is null or typszbdphk = ''typSzbDph.dphOsv'' or (
              iddphmducet is not null and iddphdalucet is not null))'
        into _polOk;
      if (_polOk = _polozek) then
        _doklOk := 1;
      end if;
    end if;
  end if;
  
  return (_doklOk = 1);
end; 
$$ LANGUAGE plpgsql;


-- projeveni zmeny ve funkci isDokladZuctovan do minulosti
update dDoklFak set zuctovano = false where storno = true;
update dDoklInt set zuctovano = false where storno = true;
update dDoklSklad set zuctovano = false where storno = true;

-- oprava triggeru nastavujicich zauctovano na dokladech - trigger je after zmeny je treba ulozit
CREATE OR REPLACE FUNCTION ddoklfakupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklFak, 'Fak') into _zuctovano;
  
  if (NEW.zuctovano <> _zuctovano) then
    update ddoklfak set zuctovano = _zuctovano where idDoklFak = NEW.idDoklFak;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION ddoklintupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklInt, 'Int') into _zuctovano;
  
  if (NEW.zuctovano <> _zuctovano) then
    update ddoklint set zuctovano = _zuctovano where idDoklInt = NEW.idDoklInt;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION ddoklskladupdatezauctovano()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _zuctovano boolean;
begin
  select isDokladZuctovan(NEW.idDoklSklad, 'Sklad') into _zuctovano;
  
  if (NEW.zuctovano <> _zuctovano) then
    update ddoklsklad set zuctovano = _zuctovano where idDoklSklad = NEW.idDoklSklad;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;



-- funkce pro prepocet obratu
CREATE OR REPLACE FUNCTION prepoctiobraty(idUcetObd integer)
  RETURNS void AS $$
declare
  platiOd date;
  platiDo date;
begin
  select platioddata, platidodata into platiOd, platiDo from nucetobdobi where iducetobdobi = idUcetObd;

  if (platiOd is not null and platiDo is not null) then
    delete from uobraty where iducetobdobi = idUcetObd and obdobi > 0;
    
    -- zauctovani polozek faktur
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred,
         P.sumZkl, P.sumZklMen) from dpolfak as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
     
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred,
         P.sumDph, P.sumDphMen) from dpolfak as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
       
     -- zauctovani polozek internich dokladu
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred,
         P.sumZkl, P.sumZklMen) from dpolint as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
       
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred,
         P.sumDph, P.sumDphMen) from dpolint as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
                                
     -- zauctovani polozek skladovych dokladu                     
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred,
         P.sumZkl, P.sumZklMen) from dpolsklad as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
       
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred,
         P.sumDph, P.sumDphMen) from dpolsklad as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
    
     -- danove naklady
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, L.idMeny, P.idStred, 
          P.sumZkl, P.sumZklMen) from mdannakl as P left outer join mleasing as L on L.idleasing = P.idleasing 
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
          
     -- danove odpisy
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred, 
          P.sumZkl, 0.0) from mdanodpisy as P
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
     
     -- splatkovy kalendar
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, L.idMeny, P.idStred, 
          P.sumZkl, P.sumZklMen) from msplatkal as P left outer join mleasing as L on L.idleasing = P.idleasing 
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
          
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idDphMdUcet, P.idDphDalUcet, L.idMeny, P.idStred, 
          P.sumDph, P.sumDphMen) from msplatkal as P left outer join mleasing as L on L.idleasing = P.idleasing 
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
     
     -- ucetni odpisy
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred, 
          P.sumZkl, 0.0) from mucetodpisy as P
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
     
     -- udalosti
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred, 
          P.sumZkl, 0.0) from mudalosti as P
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
       
  end if;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.prepoctiobraty(integer) OWNER TO %firmaRole%;


-- zvyseni cisla verze db
insert into dbversion(iddbversion, dbversion, needreimport)
        values (nextval('seq_dbversion'), 29, 't');