------------------------------------------------------------------------------------------
-- Bug 19060 – [WinStrom] Výdejka z kusovníku není zaúčtována na správné středisko
------------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION dpolskladinsertkusovnik()
  RETURNS trigger AS
$BODY$
declare
    _typPohybuK varchar;
    _typZasobyK varchar;
    _idKusovnik integer;
    _iducetobdobi integer;
    _stavNaSklade numeric;
    _sklKarta record;
    _cenik record;
    _sklzapor boolean;
    _idVydejPolSklad integer;
    _idParovanyDoklad integer;
    potKus record;
    _parKusMnoz numeric;
    _i integer;
    _kurz numeric;
    _isCiziMena boolean;
    _idMeny integer;
    _mnozMj numeric;
    _cenaMj numeric;
    _sumZkl numeric;
    _newSumZkl numeric;
    _sumZklMen numeric;
    _idSkladPotKus integer;
    _idProtiUcetDoklad integer;
    _idStred integer;
    _kopStred boolean;
begin
  -- napriklad pokud se jedna o prevodku mezi sklady nechceme z prijemky vytvaret prevodku
  if (NEW.prevodka = true) then
    return NEW;
  end if;

  -- typem pohybu musi byt prijem
  select typpohybuk into _typPohybuK from ddoklsklad d where d.iddoklsklad = NEW.iddoklsklad;
  if (_typPohybuK <> 'typPohybu.prijem') then
    return NEW;
  end if;

  -- idcenik odkazuje na neco co je soucasti kusovniku.
  select ckus.idkusovnik into _idKusovnik from ckusovnik ckus where ckus.idcenik = NEW.idcenik
    and exists (select 1 from ckusovnik cPot where cPot.idotec = ckus.idkusovnik);
  if (_idKusovnik is null) then
    return NEW;
  end if;

  -- vytvorime nebo nalezneme zparovany doklad skl vydej
  select findparovanydoklsklad(NEW.iddoklsklad, NEW.modul, 'typVazbyDokl.hlavaSklKus') into _idParovanyDoklad;
  if (_idParovanyDoklad is null) then
    select createdoklsklad(NEW.idpolsklad, NEW.iddoklsklad, NEW.modul) into _idParovanyDoklad;
  end if;

  select isDoklSkladMenaCizi(_idParovanyDoklad, NEW.datVyst) into _isCiziMena;
  select getkurzfromdoklad(_idParovanyDoklad, NEW.datVyst) into _kurz;
  select idMeny into _idMeny from dDoklSklad where idDoklSklad = _idParovanyDoklad;

  _i = 0;
  _newSumZkl = 0.0;

  perform putglotxvariable('zurnalTxName', 'rozpadKusovniku');

  -- pro vsechny kusovniky, ktere jsou pod nasim kusovnikem a jsou skladove nebo vyroba
  for potKus in (select * from ckusovnik cPot where cPot.idotec = _idKusovnik) loop
    select * into _cenik from cCenik where idCenik = potKus.idCenik;

    select typZasobyK into _typZasobyK from cCenik cen where cen.idCenik = potKus.idCenik;
    if (_typZasobyK <> 'typZasoby.skladem' and _typZasobyK <> 'typZasoby.vyrobek') then
        _newSumZkl = _newSumZkl + _cenik.nakupcena * potKus.mnoz;
        continue;
    end if;
    _i = _i + 1;

    select findIdUcetniObdobiKDatu(NEW.datVyst) into _iducetobdobi;
    -- vybereme skladovou kartu podle preferovaneho skladu
    select null into _sklKarta;
    _idSkladPotKus = null;

    select getPrefIdSkladu(
	(select idstred from ddoklsklad where iddoklsklad = NEW.iddoklsklad),
	_cenik.idskupzboz, potKus.idcenik) into _idSkladPotKus;

    if (_idSkladPotKus is not null) then
	select * into _sklKarta  from skarty where idcenik = potKus.idcenik and idtypdokl = _idSkladPotKus and iducetobdobi = _iducetobdobi;
    end if;

    -- sklad karta nebyla nalezena podle preferovaneho skladu, vezmeme sklad stejny jako z vytvarene polozky
    if (_sklKarta is null) then
	select * into _sklKarta from skarty where idcenik = potKus.idcenik and idtypdokl = NEW.idskladu and iducetobdobi = _iducetobdobi;
	select NEW.idStred into _idStred;
	select NEW.kopstred into _kopStred;
    else
	select idstred into _idStred from dtypdokl d where _sklKarta.idtypdokl = d.idtypdokl;
	select false into _kopStred;
    end if;

    -- skladovou
    if (_sklKarta is null) then
	raise exception 'Nebyla nalezena skladova karta k polozce ceniku %. Definujte mapovani skladu pro tuto polozku nebo ji pridejte na sklad %',
		_cenik.kod, (select kod from dtypdokl where idtypdokl = NEW.idskladu);
    end if;

    select mnoz into _parKusMnoz from ckusovnik where idkusovnik = _idKusovnik;

   _mnozMj = (potKus.mnoz / _parKusMnoz) * NEW.mnozMj;
   _cenaMj = _sklKarta.prumCenaTuz / _kurz;
   _sumZkl = _mnozMj * _sklKarta.prumCenaTuz;
   _newSumZkl = _newSumZkl + _sumZkl;

   if(_isCiziMena) then
     _sumZklMen = _mnozMj * _cenaMj;
   else
     _sumZklMen = 0.0;
   end if;

    -- TODO doplnit sarze a expirace
    select dgetstavnasklade(_sklKarta.idkarty, NEW.datVyst, -1, NEW.idfirmy, NULL, NULL) into _stavNaSklade;

    -- osetreni zda je povoleny zaporny stav na sklade
    if (_stavNaSklade - _mnozMj < 0) then
        select sklzapor into _sklzapor from nNastav where NEW.datVyst >= PlatiOdData order by PlatiOdData desc limit 1;
        if (_sklzapor is null) then
            select sklzapor into _sklzapor from nNastav where PlatiOdData is null limit 1;
        end if;
        if _sklzapor is null or _sklzapor = false then
            raise exception '0002:sPreporctiSkladKartu: neni povolen zaporny stav skladu';
        end if;
    end if;

    select nextval('seq_dpolsklad'::regclass) into _idVydejPolSklad;
    select idprotiucet into _idProtiUcetDoklad from ddoklsklad where iddoklsklad = _idParovanyDoklad;

    insert into dpolsklad(idPolSklad, idcenik, idskladu, idkarty, datVyst, datUcto, modul, typPolozkyK, idDoklSklad,
        mnozMj, cenaMj, sumZkl, sumZklMen, idZklMdUcet, idZklDalUcet, kopZklMdUcet, kopZklDalUcet, baleniid, mnozBaleni,
        nazev, nazevA, nazevB, nazevC, sarze, expirace, cisRad, ucetni, idStred, idZakazky, idMeny, kopstred)
    values (_idVydejPolSklad, potKus.idcenik, _sklKarta.idtypdokl, _sklKarta.idkarty, NEW.datVyst, NEW.datUcto, NEW.modul, NEW.typpolozkyk, _idParovanyDoklad,
        _mnozMj, _cenaMj, _sumZkl, _sumZklMen, _idProtiUcetDoklad, NEW.idzklmducet, true, false, NEW.baleniid, NEW.mnozbaleni,
        _cenik.nazev, _cenik.nazevA, _cenik.nazevB, _cenik.nazevC, NULL, NULL, NEW.cisRad * 100 + _i, true,
        _idStred, NEW.idZakazky, _idMeny, _kopStred);

    insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK)
      values (0.0, NEW.idpolsklad, _idVydejPolSklad, 'SKL', 'SKL', false, 'typVazbyDokl.polozkaSklKus');

  end loop;

  perform removeglotxvariable('zurnalTxName');

  select getkurzfromdoklad(NEW.iddoklsklad, NEW.datvyst) into _kurz;
  _cenaMj = _newSumZkl / NEW.mnozMj / _kurz;

  select isDoklSkladMenaCizi(NEW.iddoklsklad, NEW.datVyst) into _isCiziMena;
  if (_isCiziMena) then
    _sumZklMen = NEW.cenaMj * NEW.mnozMj;
  else
    _sumZklMen = 0.0;
  end if;

  update dpolsklad set sumzkl = _newSumZkl, cenaMj = _cenaMj, sumZklMen = _sumZklMen where idpolsklad = NEW.idpolsklad;

  return NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladinsertkusovnik() OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 291, 't');