--------------------------------------------------------------------------------
-- Bug 20610 -  [WinStrom] Po zmene kodu vyrobniho cisla nelze polozku fap ulozit
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION svyrobnicislocreatehlav() RETURNS trigger AS
$BODY$
declare
  hlav record;
begin
  -- hlavicka neexistuje -> vytvorime ji
  if (NEW.idvyrobnicislohlav is NULL) then
    insert into svyrobnicislohlav(kod, idcenik)
      values (NEW.kod, NEW.idcenik);
    select currval('seq_svyrobnicislohlav') into NEW.idvyrobnicislohlav;
  -- polozka se jiz vaze na hlavicku, pro jistotu zkontrolujeme,
  -- ze mame stejny kod a idcenik
  else
    select * into hlav from svyrobnicislohlav h where h.idvyrobnicislohlav = NEW.idvyrobnicislohlav;

    -- kod noveho pohybu se musi rovnat kodu na hlavicce
    if (tg_op = 'INSERT' and NEW.kod != hlav.kod) then
      raise exception 'Kod pohybu vyrobniho cisla % se neshoduje s kodem na hlavicce %.', NEW.kod, hlav.kod;
    end if;

    -- pouzivaji se 2 vnorene if, protoze postgresql pri vyhodnocovani vyhodnocuje
    --vsechny logicke vyrazy a OLD pri insert neni nainicializovana

    -- pri aktualizaci zkontrolujeme, ze kod pohybu je stejny jako kod na hlavicce
    if (tg_op = 'UPDATE') then
      if (OLD.kod != hlav.kod) then
        raise exception 'Kod pohybu vyrobniho cisla % se neshoduje s kodem na hlavicce %.', OLD.kod, hlav.kod;
      end if;

      -- pri aktualizaci a prepsani kodu pohybu vyrobniho cisla - pozmenime kod i na hlavicce
      if (OLD.kod != NEW.kod) then
	update svyrobnicislohlav set kod = NEW.kod where idvyrobnicislohlav = NEW.idvyrobnicislohlav;
      end if;
    end if;

    if (NEW.idcenik != hlav.idcenik) then
      raise exception 'Cenik pohybu vyrobniho cisla % se neshoduje s cenikem na hlavicce %.', NEW.idcenik, hlav.idcenik;
    end if;
  end if;

  return NEW;
end
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicislocreatehlav() OWNER TO %firmaRole%;

--------------------------------------------------------------------------------
-- Bug 20626 – [WinStrom] Použitelnost hromadné objednávky pro objednání pouze u jedné firmy
--------------------------------------------------------------------------------

-- pridani sloupce
ALTER TABLE cdodavatele ADD COLUMN automgener boolean NOT NULL DEFAULT false;

-- sesynchronizovani zaznamu
create or replace function runupgrade()
    returns void as $$
declare
    _cenik record;
    _iddod integer;
begin
  for _cenik in select * from ccenik loop
    select idfirmy into _iddod from cdodavatele where idcenik = _cenik.idcenik and primarni = true limit 1;
    
    if (_iddod is not null) then
      update ccenik set iddodavatel = _iddod where idcenik = _cenik.idcenik;
    else
      if _cenik.iddodavatel is not null then
        select iddodavatel into _iddod from cdodavatele where idcenik = _cenik.idcenik and idfirmy = _cenik.iddodavatel;
        
        if (_iddod is null) then
          insert into cdodavatele (primarni, automgener, idcenik, idfirmy)
            values (true, true, _cenik.idcenik, _cenik.iddodavatel);
        else
          update cdodavatele set primarni = true where iddodavatel = _iddod;
        end if;
      end if;
    end if;
  end loop; 
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION runupgrade() OWNER TO %firmaRole%;

select runupgrade();

drop function runupgrade();


-- trigger pri pridani/uprave ceniku
CREATE OR REPLACE FUNCTION ccenikupdatedodavatel()
  RETURNS trigger AS $$
declare
  dodOld integer;
  _iddod integer;
begin
  if (getglotxvariable('dodavatelDisableTrigger') is not null) then
    return NEW;
  end if;
  perform putglotxvariable('dodavatelDisableTrigger', '1');
  
  dodOld = null;
  if (tg_op = 'UPDATE') then
    dodOld = OLD.iddodavatel;
  end if;
  
  if not compareid(NEW.iddodavatel, dodOld) then
    delete from cdodavatele where automgener = true and idcenik = NEW.idcenik;  -- smazeme stare automaticky generovane dodavatele
    update cdodavatele set primarni = false where idcenik = NEW.idcenik;        -- ostatnim nastavime ze nejsou primarni
    
    if (NEW.iddodavatel is not null) then                                       -- pokud je zvolen nejaky novy dodavatel
      select iddodavatel into _iddod from cdodavatele where idfirmy = NEW.iddodavatel and idcenik = NEW.idcenik;
      
      if (_iddod is null) then
        insert into cdodavatele (primarni, automgener, idcenik, idfirmy)
          values (true, true, NEW.idcenik, NEW.iddodavatel);                    -- neexistuje tak ho vytvorime
      else
        update cdodavatele set primarni = true where iddodavatel = _iddod;      -- zvolenemu existujicimu nastavime primarni
      end if;
    end if;
  end if;
  
  perform removeglotxvariable('dodavatelDisableTrigger');
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.ccenikupdatedodavatel() OWNER TO %firmaRole%;

CREATE TRIGGER ccenikupdatedodavatel AFTER INSERT OR UPDATE ON ccenik
  FOR EACH ROW EXECUTE PROCEDURE ccenikupdatedodavatel();

-- trigger pri pridani/uprave dodavatele
CREATE OR REPLACE FUNCTION cdodavateleupdatedodavatel()
  RETURNS trigger AS $$
begin
  if (getglotxvariable('dodavatelDisableTrigger') is not null) then
    return NEW;
  end if;
  perform putglotxvariable('dodavatelDisableTrigger', '1');

  if NEW.primarni = true then       -- pokud je primarni nastavime ho na cenik
    update ccenik set iddodavatel = NEW.idfirmy where idcenik = NEW.idcenik;
    update cdodavatele set primarni = false where primarni = true and idcenik = NEW.idcenik and iddodavatel <> NEW.iddodavatel; -- zajisteni unikatnosti primarniho
  end if;
  
  perform removeglotxvariable('dodavatelDisableTrigger');
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.cdodavateleupdatedodavatel() OWNER TO %firmaRole%;

CREATE TRIGGER cdodavateleupdatedodavatel AFTER INSERT OR UPDATE ON cdodavatele
  FOR EACH ROW EXECUTE PROCEDURE cdodavateleupdatedodavatel();

-- trigger pri smazani dodavatele
CREATE OR REPLACE FUNCTION cdodavateledeletedodavatel()
  RETURNS trigger AS $$
begin
  if (getglotxvariable('dodavatelDisableTrigger') is not null) then
    return NEW;
  end if;
  perform putglotxvariable('dodavatelDisableTrigger', '1');
  
  if OLD.primarni = true then       -- pokud je primarni nastavime na cenik prazdny
    update ccenik set iddodavatel = null where idcenik = OLD.idcenik;
  end if;
  
  perform removeglotxvariable('dodavatelDisableTrigger');
  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.cdodavateledeletedodavatel() OWNER TO %firmaRole%;

CREATE TRIGGER cdodavateledeletedodavatel AFTER DELETE ON cdodavatele
  FOR EACH ROW EXECUTE PROCEDURE ccenikupdatedodavatel();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 370, 't');
