--------------------------------------------------------------------------------
-- Bug 21323 – [WinStrom] Polozky zavazku z leasingu maji hodnotu sazby dph 0%
--------------------------------------------------------------------------------

create or replace function runupgrade() returns void as $$
declare
 _doklfak record;
 _idsnizsazby integer;
 _idzaklsazby integer;
 _idstatuleg integer;
 _snizszb numeric;
 _zaklszb numeric;
 _polfak record;
begin
  -- nacteme stat legislativy
  -- stat legislativy by se nemel menit
  select idstatuleg into _idstatuleg from nNastav limit 1;

  -- vsechny doklady (zavazky) k splatkovym kalendarum je potreba opravit
  for _doklfak in (select * from ddoklfak
	where iddoklfak in (select distinct iddoklfak from msplatkal)
		and (sumzklzakl != 0.0 or sumdphzakl != 0.0 or
		    sumzklsniz != 0.0 or sumdphsniz != 0.0)) loop

    -- nacteme si snizenou a zakladni sazbu k datu dokladu
    -- rovnou vybereme hodnoty sazeb dph
    select idsazbydph, szbdph into _idsnizsazby, _snizszb from usazbydph
	where typszbdphk = 'typSzbDph.dphSniz' and _doklfak.duzpucto >= platioddata
	and (platidodata is null or _doklfak.duzpucto <= platidodata)
	and idstatu = _idstatuleg
	order by platioddata desc
	limit 1;

    select idsazbydph, szbdph into _idzaklsazby, _zaklszb from usazbydph
	where typszbdphk = 'typSzbDph.dphZakl' and _doklfak.duzpucto >= platioddata
	and (platidodata is null or _doklfak.duzpucto <= platidodata)
	and idstatu = _idstatuleg
	order by platioddata desc
	limit 1;

    -- provedeme update dokladu
    update ddoklfak
    set szbdphzakl = _zaklszb,
      szbdphsniz = _snizszb,
      idsazbydphzakl = _idzaklsazby,
      idsazbydphsniz = _idsnizsazby
    where iddoklfak = _doklfak.iddoklfak;

    -- opravime i polozky techto dokladu
    for _polfak in (select * from dpolfak where iddoklfak = _doklfak.iddoklfak) loop
      -- je potreba ji opravit, sazba je nulova i kdyz typ neni osvobozeny
      if (_polfak.typszbdphk != 'typSzbDph.dphOsv' and _polfak.szbdph = 0.0) then
	if (_polfak.typszbdphk = 'typSzbDph.dphZakl') then
	  update dpolfak
	    set szbdph = _zaklszb
            where idpolfak = _polfak.idpolfak;
	end if;
	if (_polfak.typszbdphk = 'typSzbDph.dphSniz') then
	  update dpolfak
	    set szbdph = _snizszb
            where idpolfak = _polfak.idpolfak;
	end if;
      end if;
    end loop;

  end loop;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION runupgrade() OWNER TO %firmaRole%;

select runupgrade();

drop function runupgrade();

--------------------------------------------------------------------------------
-- Bug 21220 – [WinStrom] Přidat funkci objednání pro výrobu
--------------------------------------------------------------------------------

drop function findmnozzkusovniku(integer[], integer[], numeric[], boolean);
drop function findmnozzkusovniku(integer, integer, numeric, boolean);
drop type cenikmnoz;


create type cenikmnoz as (idcenik integer, mnoz numeric, idcenikpuv integer, idsklad integer, idstred integer, leaf boolean, uroven integer);

CREATE OR REPLACE FUNCTION findmnozzkusovniku(_idoteccenik integer, _idstred integer, _objmnoz numeric, leavesonly boolean, maxlevel integer, aktlevel integer)
  RETURNS SETOF cenikmnoz AS
$BODY$
declare
  _cenikmnoz cenikmnoz%rowtype;
  _subcenikmnoz cenikmnoz%rowtype;
  _kus record;
  _idsklad integer;
  _hasleaves boolean;
begin
  -- pro vsechny kusovniky majici za otce _idoteccenik
  for _kus in (select * from ckusovnik where idoteccenik = _idoteccenik) loop
    -- pokud nemuze byt uzel samostatnym korenem pridame ho mezi vysledky
    if (_kus.idcenik != _kus.idoteccenik) then
      _hasleaves = false;
      
      -- a pokud neni aktualni uroven urovni posledni (tj kdyz se zada maxlevel = 0 pojede se do konce, protoze rovnost nenastane)
      if (maxlevel <> aktlevel) then
        -- projdeme jeho kusovniky, ze kterych se sklada, pridame si je mezi vysledky
        for _subcenikmnoz in select * from findmnozzkusovniku(_kus.idcenik, _idstred, _kus.mnoz * _objmnoz, leavesonly, maxlevel, aktlevel + 1) loop
          _hasleaves = true;
          _subcenikmnoz.idcenikpuv := _idoteccenik;
          return next _subcenikmnoz;
        end loop;
      end if;

      -- jedna se o surovinu a chceme jenom suroviny
      -- nebo chceme vsechny
      if ((_hasleaves = false and leavesonly)
          or leavesonly = false) then
        _cenikmnoz.idcenik := _kus.idcenik;
        _cenikmnoz.mnoz := _kus.mnoz * _objmnoz;
        _cenikmnoz.idcenikpuv := _idoteccenik;
        _cenikmnoz.idstred := _idstred;
        _cenikmnoz.leaf = not _hasleaves;
        _cenikmnoz.uroven = aktlevel;
        -- nalezneme preferovany sklad
        _idsklad = getprefidskladu(_idstred, (select idskupzboz from ccenik c where c.idcenik = _kus.idcenik), _kus.idcenik);
        _cenikmnoz.idsklad = _idsklad;
        return next _cenikmnoz;
      end if;
    end if;
  end loop;
  return;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION findmnozzkusovniku(integer, integer, numeric, boolean, integer, integer) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION findmnozzkusovniku(_idoteccenik integer[], _idstred integer[], _objmnoz numeric[], leavesonly boolean, maxlevel integer)
  RETURNS SETOF cenikmnoz AS
$BODY$
declare
  _indx integer;
  _cenikmnoz cenikmnoz%rowtype;
begin
  if (array_upper(_idoteccenik, 1) != array_upper(_objmnoz, 1) or
	array_upper(_objmnoz, 1) != array_upper(_idstred, 1)) then
    raise exception 'Velikost poli musi byt shodna';
  end if;
  
  _indx = 0;
  while (_indx < array_upper(_idoteccenik, 1)) loop
    _indx = _indx + 1;
    for _cenikmnoz in select * from findmnozzkusovniku(_idoteccenik[_indx], _idstred[_indx], _objmnoz[_indx], leavesonly, maxlevel, 1) loop
      return next _cenikmnoz;
    end loop;
  end loop;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION findmnozzkusovniku(integer[], integer[], numeric[], boolean, integer) OWNER TO %firmaRole%;

---------------------------------------------------------------------------------
-- Bug 21321 – [WinStrom] chybně vytvořená příjemka u přijatého dobropisu - FIFO
---------------------------------------------------------------------------------

-- Function: svyrobnicisloupdate()
-- DROP FUNCTION svyrobnicisloupdate();
CREATE OR REPLACE FUNCTION svyrobnicisloupdate()
  RETURNS trigger AS
$BODY$    
declare 
  _sklFifo boolean; 
  _mnozMj numeric(19,6); 
  _datVyst date; 
  _idRozlozeni integer; 
  _cenaMjP numeric(19,6); 
  _kurzDokladuP numeric(19,6); 
  _kurzDokladuV numeric(19,6); 
  _idDoklSkladP integer; 
  _idDoklSkladV integer; 
  _datVystV date; 
  _cenaMjPMen numeric(19,6);
  _mnozVyrobnichCisel numeric(19,6); 
  _mnozMjPoZmene numeric(19,6);
begin 
  if(NEW.idPolSkladVydej is not null and OLD.idPolSkladVydej is null) then 
    select datVyst, mnozMj into _datVyst, _mnozMj from dPolSklad where idPolSklad = NEW.idPolSkladVydej; 
    _mnozVyrobnichCisel = 1;
    _mnozMjPoZmene = _mnozMj;
    if(_mnozMj < 0.0) then
      _mnozMjPoZmene = 0.0 - _mnozMj;
      _mnozVyrobnichCisel = -1;
    end if;
 
    _sklfifo := (select isfifosklad(_datVyst)); 
    if(_sklFifo) then 
 
      select cenamj, idDoklSklad  into _cenamjP, _idDoklSkladP from dpolsklad where(idpolsklad = NEW.idPolSkladPrijem);     
      select kurz into _kurzDokladuP from ddoklsklad where (iddoklsklad = _idDoklSkladP);    
      if (_kurzDokladuP <> 0.0) then    
	_cenamjP := _cenamjP * _kurzDokladuP;     
      end if;   
      select iddoklsklad, datVyst into _idDoklSkladV, _datVystV from dpolsklad where idpolsklad = NEW.idPolSkladVydej;   
      _KurzDokladuV := (select getKurzFromDoklad(_idDoklSkladV, _datVystV));   
      _cenamjP := _cenamjP / _kurzDokladuV; 
      _cenaMjPMen := 0.0; 
	  if(_kurzDokladuV <> 1.0) then 
        _cenaMjPMen := _cenaMjP; 
      end if; 
      if(_mnozMjPoZmene > 1) then 
        -- odecteni 1 z mnoz mj a vytvoreni nove polozky a mnozstvim 1 a 1 vyrobnim cislem 
        select nextval('seq_dpolsklad') into _idrozlozeni;                        
	update dpolsklad set mnozMJ = _mnozMj - _mnozVyrobnichCisel where idPolSklad = NEW.idPolSkladVydej; 
         
        insert into dpolsklad(idpolsklad,           mnozmj,             autozaokr,autogen,baleniid,cenjednotka,cenamj,  cenamjnakup,cenamjprodej,cisrad,datucto,datvyst,eankod,expirace,kod,kopclendph,kopdphdalucet,iddphmducet,kopklice,kopstred,kopzakazku,kopzkldalucet,kopzklmducet,mnozbaleni,modul,nazev,objednatk,objem,poznam,proczakl,rezervovat,sarze,slevadokl,slevamnoz,slevapol,stavuzivk,storno,stornopol,sumdph,sumdphmen,sumzkl,                                       sumzklmen,                       szbdph,typcenydphk,typpolozkyk,typszbdphk, typvypcenyk,ucetni,uplslevadokl,zaokrjakk,zaokrnak,idzkldalucet,iddphdalucet,idmjobjem,idmeny,idstred,idkarty,idcenik,idzklmducet,idsazbydph,idclendph,idzakazky,idmj,iddoklsklad,idskladu,idfirmy,vyrcis) 
	                     (select (_idrozlozeni),_mnozVyrobnichCisel,autozaokr,autogen,baleniid,cenjednotka,_cenamjP,cenamjnakup,cenamjprodej,cisrad,datucto,datvyst,eankod,expirace,kod,kopclendph,kopdphdalucet,iddphmducet,kopklice,kopstred,kopzakazku,kopzkldalucet,kopzklmducet,mnozbaleni,modul,nazev,objednatk,objem,poznam,proczakl,rezervovat,sarze,slevadokl,slevamnoz,slevapol,stavuzivk,storno,stornopol,sumdph,sumdphmen,_cenamjP*_kurzDokladuV * _mnozVyrobnichCisel,_cenaMjPMen * _mnozVyrobnichCisel,szbdph,typcenydphk,typpolozkyk,typszbdphk, typvypcenyk,ucetni,uplslevadokl,zaokrjakk,zaokrnak,idzkldalucet,iddphdalucet,idmjobjem,idmeny,idstred,idkarty,idcenik,idzklmducet,idsazbydph,idclendph,idzakazky,idmj,iddoklsklad,idskladu,idfirmy,vyrcis from dpolsklad where idpolsklad = NEW.idpolskladvydej); 
	insert into sfifovazby(idpolskladprijem,idpolskladvydej,kusu,idvyrobnicislo) values (NEW.idPolSkladPrijem,_idrozlozeni,1,NEW.idvyrobnicislo); 
	insert into svazbypolsklad(idpolskladzdroj,idpolskladcil) values (NEW.idPolSkladVydej, _idrozlozeni); 
	NEW.idPolSkladVydej = _idRozlozeni; 
      else 
        -- nastaveni vyrobniho cisla na a vytvoreni vazby 
        --update sfifovazby set idvyrobnicislo = NEW.idvyrobnicislo, kusu = 1, idpolskladprijem = NEW.idpolskladprijem where idpolskladvydej = NEW.idpolskladvydej and idvyrobnicislo = -1; 
        insert into sfifovazby(idpolskladprijem,idpolskladvydej,kusu,idvyrobnicislo) values (NEW.idPolSkladPrijem,NEW.idPolSkladVydej,1,NEW.idvyrobnicislo); 
        --select cenaMj into _cenaMjP from dpolsklad where idPolSklad = NEW.idPolSkladPrijem; 
        update dpolsklad set cenaMJ = _cenaMjP, sumZkl = _cenamjP*_kurzDokladuV*_mnozVyrobnichCisel, sumZklMen = _cenaMjPMen*_mnozVyrobnichCisel where idPolSklad = NEW.idPolSkladVydej; 
      end if; 
      -- 
    else 
      insert into sfifovazby(idpolskladprijem,idpolskladvydej,kusu,idvyrobnicislo) values (NEW.idPolSkladPrijem,NEW.idPolSkladVydej,1,NEW.idvyrobnicislo); 
    end if; 
  elsif (NEW.idPolSkladVydej is null and OLD.idPolSkladVydej is not null) then 
    -- pokud se idPolSkladVydej nastavi na null z nejake hodnoty musi se smazat i fifo vazba s timto vyrobnim cislem 
    delete from sfifovazby where idvyrobnicislo = NEW.idVyrobniCislo; 
  end if; 
  execute sprepoctisumydokladu(_idDoklSkladV); 
  return NEW; 
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicisloupdate() OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 401, 't');
