--------------------------------------------------------------------------------
-- Bug 21247 – [WinStrom] Upravit účtování, aby se nechalo účtovat i ZP a SP odlišně pro společníky
--------------------------------------------------------------------------------

-- jeste doplnime skupinu osob
ALTER TABLE pskuposob ADD COLUMN idtuozavdan integer;

ALTER TABLE pskuposob ADD CONSTRAINT pskuposob_idtuozavdan FOREIGN KEY (idtuozavdan)
  REFERENCES utypucop (idtypucop) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX pSkupOsob_IdTuoZavDan_index ON pSkupOsob USING btree (IdTuoZavDan);

-- rozsirime pracovni pomer
ALTER TABLE ppracpom ADD COLUMN idtuozavsoczam integer;
ALTER TABLE ppracpom ADD COLUMN idtuozavsocfir integer;
ALTER TABLE ppracpom ADD COLUMN idtuozavzdrzam integer;
ALTER TABLE ppracpom ADD COLUMN idtuozavzdrfir integer;
ALTER TABLE ppracpom ADD COLUMN idtuozavdan integer;

ALTER TABLE ppracpom ADD CONSTRAINT ppracpom_idtuozavsoczam FOREIGN KEY (idtuozavsoczam)
  REFERENCES utypucop (idtypucop) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE ppracpom ADD CONSTRAINT ppracpom_idtuozavsocfir FOREIGN KEY (idtuozavsocfir)
  REFERENCES utypucop (idtypucop) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE ppracpom ADD CONSTRAINT ppracpom_idtuozavzdrzam FOREIGN KEY (idtuozavzdrzam)
  REFERENCES utypucop (idtypucop) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE ppracpom ADD CONSTRAINT ppracpom_idtuozavzdrfir FOREIGN KEY (idtuozavzdrfir)
  REFERENCES utypucop (idtypucop) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE ppracpom ADD CONSTRAINT ppracpom_idtuozavdan FOREIGN KEY (idtuozavdan)
  REFERENCES utypucop (idtypucop) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX pPracPom_IdTuoZavSocZam_index ON pPracPom USING btree (IdTuoZavSocZam);
CREATE INDEX pPracPom_IdTuoZavSocFir_index ON pPracPom USING btree (IdTuoZavSocFir);
CREATE INDEX pPracPom_IdTuoZavZdrZam_index ON pPracPom USING btree (IdTuoZavZdrZam);
CREATE INDEX pPracPom_IdTuoZavZdrFir_index ON pPracPom USING btree (IdTuoZavZdrFir);
CREATE INDEX pPracPom_IdTuoZavDan_index ON pPracPom USING btree (IdTuoZavDan);

--------------------------------------------------------------------------------
--Bug 20323 – [WinStrom] "Předrezervace" - nastavovani korektniho casu vzniku
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION rezervujPolObch(_idCenik integer, _idSklad integer, _idPolObch integer, _mj numeric)
  RETURNS void AS $$
declare
  polObch record;
  _mnoz numeric(19,6);
  _idUcetObd integer;
  _rezer record;
  _idFirma integer;
  _idUziv integer;
begin
  select * into _rezer from srezervace where idPolObch = _idPolObch and idCenik = _idCenik and idSklad = _idSklad;
  
  _mnoz = _mj;
  if _rezer.idrezervace is not null then
    _mnoz = _mnoz + _rezer.mnozstvi;
  end if;
  
  if _mnoz > 0 then             -- pokud ma neco byt zarezervovane
    select * into polObch from dPolObch where idPolObch = _idPolObch;
    
    if _rezer.idrezervace is null then      -- pokud rezervace neexistuje vytvorime ji
      select idFirmy, idUziv into _idFirma, _idUziv from dDoklObch where idDoklObch = polObch.idDoklObch;
      
      insert into srezervace (lastupdate, idpolobch, iduziv, idfirmy, idcenik, idsklad, mnozstvi, datumod)
          values (now(), _idPolObch, _idUziv, _idFirma, _idCenik, _idSklad, _mnoz, date_trunc('minute', now()));
    else
      update srezervace set mnozstvi = _mnoz where idrezervace = _rezer.idRezervace;
    end if;
  else
    if _rezer.idrezervace is not null then    -- pokud rezervace byt nema a je smazeme ji
      delete from srezervace where idrezervace = _rezer.idRezervace;
    end if;
  end if;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.rezervujPolObch(integer, integer, integer, numeric) OWNER TO %firmaRole%;

-- oprava existujicich dat
update srezervace set datumod = date_trunc('minute', datumod) where datumod <> date_trunc('minute', datumod);

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 428, 't', %appVersion%, clock_timestamp());
