--------------------------------------------------------------------------------
-- Bug 29251 – [winstrom] mzdy - výpočet mzdy při ukončení PP
--------------------------------------------------------------------------------

-- trigger generujici mzdove slozky pri uprave pracovnich pomeru
CREATE OR REPLACE FUNCTION ppracpomhlavupdatemzdsloz()
  RETURNS trigger AS $$
declare
  nullcount integer;
  _rok integer;
  _mesic integer;
  _idSmena integer;
  pregenSmenuRok integer;
  pregenSmenuMesic integer;
  pregenSmenuOd date;
  pregenSmenuDo date;
begin
  -- vymazeme nadbytecne slozky pro pracovni pomer
  if (NEW.aktivniod is not null) then
    _rok = date_part('year', NEW.aktivniod);
    _mesic = date_part('month', NEW.aktivniod);
    
    delete from pmzdsloz where idpracpom = NEW.idpracpomhlav and (rok < _rok or (rok = _rok and mesic < _mesic));
    delete from psmeny where idpracpom = NEW.idpracpomhlav and (rok < _rok or (rok = _rok and mesic < _mesic));
  end if;
  
  if (NEW.aktivnido is not null) then
    _rok = date_part('year', NEW.aktivnido);
    _mesic = date_part('month', NEW.aktivnido);
    
    delete from pmzdsloz where idpracpom = NEW.idpracpomhlav and (rok > _rok or (rok = _rok and mesic > _mesic));
    delete from psmeny where idpracpom = NEW.idpracpomhlav and (rok > _rok or (rok = _rok and mesic > _mesic));
  end if;
  
  -- vymazeme nadbytecne spolecne slozky
  select count(*) into nullcount from ppracpomhlav where idosoba = NEW.idosoba and aktivniod is null;
  if nullcount = 0 then
    select date_part('year', min(aktivniod)), date_part('month', min(aktivniod)) into _rok, _mesic from ppracpom where idosoba = NEW.idosoba;
    
    delete from pmzdsloz where idosoba = NEW.idosoba and (rok < _rok or (rok = _rok and mesic < _mesic));
  end if;
  
  select count(*) into nullcount from ppracpomhlav where idosoba = NEW.idosoba and aktivnido is null;
  if nullcount = 0 then
    select date_part('year', max(aktivnido)), date_part('month', max(aktivnido)) into _rok, _mesic from ppracpom where idosoba = NEW.idosoba;
    
    delete from pmzdsloz where idosoba = NEW.idosoba and (rok > _rok or (rok = _rok and mesic > _mesic));
  end if;
  
  -- vygenerujeme chybejici
  perform pgeneratemzdslozforpracpom(NEW.idpracpomhlav);
  
  if (tg_op = 'UPDATE') then       -- pri zmene aktivni od/do zjistime jesteli neni potreba pregenerovat nejaky mesic, kvuli dobe kdy nepracuje
    -- zkontrolujeme zda se nezmenil zacatek pomeru a neni treba neco pregenerovat
    if (comparedate(NEW.zacatek, OLD.zacatek) = false) then
      pregenSmenuRok = 0;
      pregenSmenuMesic = 0;
      
      if (NEW.zacatek is null) then
        if extract(day from OLD.zacatek) > 1 then
          pregenSmenuRok = extract(year from OLD.zacatek);
          pregenSmenuMesic = extract(month from OLD.zacatek);
        end if;
      end if;
      if (OLD.zacatek is null) then
        if extract(day from NEW.zacatek) > 1 then
          pregenSmenuRok = extract(year from NEW.zacatek);
          pregenSmenuMesic = extract(month from NEW.zacatek);
        end if;
      end if;
      if NEW.zacatek is not null and OLD.zacatek is not null then
        if (extract(month from NEW.zacatek) = extract(month from OLD.zacatek) and extract(year from NEW.zacatek) = extract(year from OLD.zacatek)) then
          pregenSmenuRok = extract(year from NEW.zacatek);       -- zmenil se den pocatku ale mesic zustal
          pregenSmenuMesic = extract(month from NEW.zacatek);
        else
          if (extract(day from OLD.zacatek) > 1 and (extract(month from NEW.zacatek) < extract(month from OLD.zacatek) or extract(year from NEW.zacatek) < extract(year from OLD.zacatek))) then
            pregenSmenuRok = extract(year from NEW.zacatek);    -- stary den pocatku neni 1 a mesic se posunul v case dozadu - tj mesic existule ale uz je tam celej
            pregenSmenuMesic = extract(month from NEW.zacatek);
          end if;
          if (extract(day from NEW.zacatek) > 1 and (extract(month from NEW.zacatek) > extract(month from OLD.zacatek) or extract(year from NEW.zacatek) > extract(year from OLD.zacatek))) then
            pregenSmenuRok = extract(year from NEW.zacatek);    -- novy den pocatku neni 1 a mesic se posunul v case dopredu - tj mesic existuje ale meni se
            pregenSmenuMesic = extract(month from NEW.zacatek);
          end if;
        end if;
      end if;
      
      if pregenSmenuMesic > 0 then
        select idsmeny into _idSmena from psmeny where idpracpom = NEW.idpracpomhlav and rok = pregenSmenuRok and mesic = pregenSmenuMesic;
        if _idSmena is not null then
          perform pcreatemzdslozforsmeny(_idSmena);
        end if;
      end if;
    end if;
    
    -- zkontrolujeme zda se nezmenil konec pomeru a neni treba neco pregenerovat
    if (comparedate(NEW.konecpomeru, OLD.konecpomeru) = false) then
      pregenSmenuOd = null;
      pregenSmenuDo = null;
      if (NEW.aktivnido is not null and (OLD.aktivnido is null or NEW.aktivnido <= OLD.aktivnido)) then
        pregenSmenuDo = NEW.aktivnido;
      end if;
      if (OLD.aktivnido is not null and (NEW.aktivnido is null or OLD.aktivnido < NEW.aktivnido)) then
        pregenSmenuDo = OLD.aktivnido;
      end if;

      if (NEW.konecpomeru is null) then
        pregenSmenuOd = date_trunc('month', OLD.konecpomeru);
        if iskonecmesice(OLD.konecpomeru) = false then
          pregenSmenuOd = pregenSmenuOd + interval '1 month';
        end if;
      end if;
      if (OLD.konecpomeru is null) then
        pregenSmenuOd = date_trunc('month', NEW.konecpomeru);
        if iskonecmesice(NEW.konecpomeru) = false then
          pregenSmenuOd = pregenSmenuOd + interval '1 month';
        end if;
      end if;
      if NEW.konecpomeru is not null and OLD.konecpomeru is not null then
        if (extract(month from NEW.konecpomeru) = extract(month from OLD.konecpomeru) and extract(year from NEW.konecpomeru) = extract(year from OLD.konecpomeru)) then
          pregenSmenuOd = date_trunc('month', NEW.konecpomeru);                 -- zmenil se den konce ale mesic zustal
          pregenSmenuDo = pregenSmenuOd;
        else
          if (NEW.konecpomeru < OLD.konecpomeru) then
            pregenSmenuOd = NEW.konecpomeru;
            pregenSmenuDo = date_trunc('month', OLD.konecpomeru);
          else
            pregenSmenuOd = OLD.konecpomeru;
            pregenSmenuDo = date_trunc('month', NEW.konecpomeru);
          end if;
          
          if iskonecmesice(pregenSmenuOd) then
            pregenSmenuOd = pregenSmenuOd + interval '1 day';
          else
            pregenSmenuOd = date_trunc('month', pregenSmenuOd);
          end if;
        end if;
      end if;
      
      if pregenSmenuOd is not null then
        perform pcreatemzdslozforsmeny(idsmeny) from psmeny where idpracpom = NEW.idpracpomhlav and 
          pregenSmenuOd <= date('' || rok || '-' || mesic || '-01') and
          (pregenSmenuDo is null or pregenSmenuDo >= date('' || rok || '-' || mesic || '-01'));
      end if;
    end if;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.ppracpomhlavupdatemzdsloz() OWNER TO %firmaRole%;

-- pro smeny se zadanym id vygeneruje mzdovou slozku pracovni doby
CREATE OR REPLACE FUNCTION pcreatemzdslozforsmeny(_idSmeny integer)
  RETURNS void AS $$
declare
  SM record;
  SMnext record;
  _hodin numeric;
  _dnuPrac numeric;
  _hodinPoc numeric;
  _dnuPracPoc numeric;
  _konecMes date;
  _okolMes date;
  _denOd integer;
  _denDo integer;
  _denOdPoc integer;
  _denDoPoc integer;
  _pocatek boolean;
  _jizNeprac boolean;
  _aktivOd date;
  _aktivDo date;
  i integer;
  smeny varchar[];
  smena varchar[];
  _aktSmena varchar;
  pocSmena integer;
  konSmena integer;
  denSmena integer;
  _idcisMzd integer;
  _idmzdsloz integer;
  _aktHodin numeric;
  _svatek boolean;
  _dnuSvat numeric;
  _hodinSvat numeric;
  _dnuSvatPoc numeric;
  _hodinSvatPoc numeric;
begin
  select * into SM from psmeny where idsmeny = _idsmeny;
  _hodin = 0;
  _dnuPrac = 0;
  _hodinPoc = 0;
  _dnuPracPoc = 0;
  _dnuSvat = 0;
  _hodinSvat = 0;
  _dnuSvatPoc = 0;
  _hodinSvatPoc = 0;
  _konecMes = date(SM.rok || '-' || SM.mesic || '-01') + interval '1 month' - interval '1 day';
  _denOd = 1;
  _denDo = extract (day from _konecMes);
  _denOdPoc = 1;
  _denDoPoc = _denDo;
  _jizNeprac = false;
  
  select zacatek, konecpomeru into _aktivOd, _aktivDo from ppracpomhlav where idpracpomhlav = SM.idpracpom;
  
  if (_aktivOd is not null) then
    if (SM.mesic = extract(month from _aktivOd) and SM.rok = extract(year from _aktivOd)) then
      _denOdPoc = extract (day from _aktivOd);
    end if;
  end if;
  if (_aktivDo is not null) then
    if (SM.mesic = extract(month from _aktivDo) and SM.rok = extract(year from _aktivDo)) then
      _denDoPoc = extract (day from _aktivDo);
    end if;
    if (date('' || SM.rok || '-' || SM.mesic || '-01') > _aktivDo) then
      _jizNeprac = true;
    end if;
  end if;
  _pocatek = (_denOd <> _denOdPoc or _denDo <> _denDoPoc);
  
  if (_jizNeprac = false and SM.planovane is not null and length(SM.planovane) > 0) then
    smeny = string_to_array(SM.planovane,';');
    _aktSmena = '';
    
    for i in array_lower(smeny,1) .. array_upper(smeny,1) loop  -- prochazime smeny
      smena = string_to_array(smeny[i],':');
      if (smena[1] <> '0' and smena[1] <> 's0') then                     -- zbytek smeny z minuleho mesice nas nezajima
        pocSmena = smena[2];
        konSmena = smena[3];
        
        if (_aktSmena <> smena[1]) then
          denSmena = pocSmena / (24 * 60) + 1;    -- zjistime den
          _svatek = (strpos(smena[1], 's') > 0);    -- zjistime jestli jde o svatek
          
          if (_svatek) then
            _dnuSvat = _dnuSvat + 1;  -- pripocteme svatek kdy mel pracovat ale flakal se
            if (denSmena < _denOdPoc or denSmena > _denDoPoc) then
              _dnuSvatPoc = _dnuSvatPoc + 1;
            end if;
          else
            _dnuPrac = _dnuPrac + 1;  -- pripocteme pracovni den
            if (denSmena < _denOdPoc or denSmena > _denDoPoc) then
              _dnuPracPoc = _dnuPracPoc + 1;
            end if;
          end if;
          _aktSmena = smena[1];
        end if;
        
        _aktHodin = cast((konSmena - pocSmena + 1) as numeric) / 60;
        
        if (_svatek) then
          _hodinSvat = _hodinSvat + _aktHodin;  -- pripocteme svatecni hodiny
          if (denSmena < _denOdPoc or denSmena > _denDoPoc) then
            _hodinSvatPoc = _hodinSvatPoc + _aktHodin;
          end if;
        else
          _hodin = _hodin + _aktHodin;    -- pripocteme pracovni hodiny
          if (denSmena < _denOdPoc or denSmena > _denDoPoc) then
            _hodinPoc = _hodinPoc + _aktHodin;
          end if;
        end if;
      else
        _okolMes = date(SM.rok || '-' || SM.mesic || '-01') - interval '1 month';
        select pcreatemzdslozforsmeny(idsmeny)from psmeny where idpracpom = sm.idpracpom and 
               rok = extract(year from _okolMes) and mesic = extract(month from _okolMes);
      end if;
    end loop;
  end if;
  
  -- podivame se o mesic dal jestli tam nemame kousicek
  /*_okolMes = date(SM.rok || '-' || SM.mesic || '-01') + interval '1 month';
  select * into SMnext from psmeny where idpracpom = sm.idpracpom and 
               rok = extract(year from _okolMes) and mesic = extract(month from _okolMes) limit 1;
  if (SMnext.idsmeny is not null) then
    if (SMnext.planovane is not null and length(SMnext.planovane) > 0) then
      smeny = string_to_array(SMnext.planovane,';');
      
      for i in array_lower(smeny,1) .. array_upper(smeny,1) loop  -- prochazime smeny
        smena = string_to_array(smeny[i],':');
        _aktSmena = smena[1];
      
        if (_aktSmena = '0' or _aktSmena = 's0') then
          pocSmena = smena[2];
          konSmena = smena[3];
          _aktHodin = (konSmena - pocSmena + 1) / 60;
          if (_aktSmena = '0') then
            _hodin = _hodin + _aktHodin;
          else
            _hodinSvat = _hodinSvat + _aktHodin;
          end if;
        end if;
      end loop;  
    end if;
  end if;*/
  
  select idcismzdsloz into _idcisMzd from pcismzdsloz where typslozkyk = 'typSlozky.kopie' and skupslozkyk = 'skupinaSlozky.doba';
  select idmzdsloz into _idmzdsloz from pmzdsloz where idpracpom = SM.idpracpom and rok = SM.rok and mesic = SM.mesic 
      and idcismzdsloz = _idcisMzd;
  
  if (_idmzdsloz is null) then      -- slozka neexistuje vytvorime ji
    select pcreatemzdslozfromcis(_idcisMzd, SM.idpracpom, null, SM.rok, SM.mesic) into _idmzdsloz;
  end if;
  
  update pmzdsloz set dnyKal = (_denDo-_denOd+1), dnyPrac = _dnuPrac, hodiny = _hodin, dnySvatek = _dnuSvat, hodinySvatek = _hodinSvat where idmzdsloz = _idmzdsloz;

  
  select idcismzdsloz into _idcisMzd from pcismzdsloz where typslozkyk = 'typSlozky.inicializace' and skupslozkyk = 'skupinaSlozky.doba';
  if (_idcisMzd is not null) then
    select idmzdsloz into _idmzdsloz from pmzdsloz where idpracpom = SM.idpracpom and rok = SM.rok and mesic = SM.mesic
      and idcismzdsloz = _idcisMzd;

    if (_jizNeprac = false and _pocatek = true) then
      if (_idmzdsloz is null) then      -- slozka neexistuje vytvorime ji
        select pcreatemzdslozfromcis(_idcisMzd, SM.idpracpom, null, SM.rok, SM.mesic) into _idmzdsloz;
      end if;

      update pmzdsloz set dnyKal = (_denDo-_denOd - _denDoPoc+_denOdPoc), dnyPrac = _dnuPracPoc, hodiny = _hodinPoc, dnySvatek = _dnuSvatPoc, hodinySvatek = _hodinSvatPoc where idmzdsloz = _idmzdsloz;
    else
      if (_idmzdsloz is not null) then      -- slozka neexistuje smazeme ji
        delete from pmzdsloz where idmzdsloz = _idmzdsloz;
      end if;
    end if;
  end if;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.pcreatemzdslozforsmeny(integer) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 608, 't', %appVersion%, clock_timestamp());
