--------------------------------------------------------------------------------
-- Bug 30205 – [WinStrom] Zaúčtovaný doklad v kase nejde převézt na fakturu
--------------------------------------------------------------------------------

-- trigger volaný při smazání vazby prodejky
CREATE OR REPLACE FUNCTION dvazebtabdeletevazbaprodejky()
  RETURNS trigger AS $$
begin
  if OLD.typvazbyk = 'typVazbyDokl.zaucProdejky' then
    update ddoklfak set stavuzivk = null where iddoklfak = OLD.idtaba and stavuzivk is not null;
  end if;
  
  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dvazebtabdeletevazbaprodejky() OWNER TO %firmaRole%;

-- trigger volaný při aktualizaci faktury - pokud se jedná o prodejku, je zaúčtovaná a dojde ke změně ovlivňující zaúčtování, odnastaví stav zaúčtováno
CREATE OR REPLACE FUNCTION ddoklfaktupdateprodejky()
  RETURNS trigger AS $$
begin
  if (OLD.prodejka = true and NEW.prodejka = false) then      -- delame z prodejky fakturu
    NEW.stavuzivk = null;
  end if;
  
  if (NEW.prodejka = true and NEW.stavuzivk = 'stavUziv.zauctovano' and OLD.stavuzivk = 'stavUziv.zauctovano') then
    if (NEW.sumcelkem <> OLD.sumcelkem or NEW.sumcelkemmen <> OLD.sumcelkemmen or NEW.datvyst <> OLD.datvyst) then
      NEW.stavuzivk = 'stavUziv.zaucZmena';
    end if;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.ddoklfaktupdateprodejky() OWNER TO %firmaRole%;

-- trigger volaný po aktualizaci faktury - pokud se jednalo o prodejku a nyní již ne, smaže vazbu zaúčtování
CREATE OR REPLACE FUNCTION ddoklfaktupdateprodejkyafter()
  RETURNS trigger AS $$
begin
  if (OLD.prodejka = true and NEW.prodejka = false) then      -- delame z prodejky fakturu
    delete from dvazebtab where idtaba = NEW.iddoklfak and modula = 'FAV' and typvazbyk = 'typVazbyDokl.zaucProdejky';  -- zrusime vazbu prodejky
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.ddoklfaktupdateprodejkyafter() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku faktur
CREATE TRIGGER ddoklfaktupdateprodejkyafter AFTER UPDATE ON ddoklfak
  FOR EACH ROW EXECUTE PROCEDURE ddoklfaktupdateprodejkyafter();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 632, 't', %appVersion%, clock_timestamp());
