--------------------------------------------------------------------------------
-- Bug 33205 – [FlexiBee] Prodloužit uživatelské jméno na 50 znaků
--------------------------------------------------------------------------------

ALTER TABLE wuzivatel ALTER kod TYPE character varying(255);
ALTER TABLE wuzivatel ADD dbusername character varying(63);
CREATE UNIQUE INDEX wuzivatel_dbusername_index ON wuzivatel USING btree (dbusername);

ALTER TABLE wzurnal ALTER uziv TYPE character varying(63);
ALTER TABLE changelog ALTER uzivatel TYPE character varying(63);


CREATE OR REPLACE FUNCTION logzurnal() RETURNS trigger AS $$
    # PL/Tcl function body

    spi_exec "SELECT pg_attribute.attname AS pk
        FROM pg_index, pg_class, pg_attribute
        WHERE
          pg_class.oid = '[ quote $TG_table_name ]'::regclass AND
          indrelid = pg_class.oid AND
          pg_attribute.attrelid = pg_class.oid AND
          pg_attribute.attnum = any(pg_index.indkey)
          AND indisprimary"

    if {$TG_op == "INSERT" || $TG_op == "UPDATE"} {
        set pkval [ quote $NEW([ quote $pk ]) ]
    } else {
        set pkval [ quote $OLD([ quote $pk ]) ]
    }

    spi_exec "SELECT now() AS log_time"

    spi_exec "SELECT getGloTxVariable('zurnalTxName') AS tx_name"
    if {![ info exists tx_name ]} {
        set tx_name null
    } else {
        set tx_name '[ quote $tx_name ]'
    }

    spi_exec "SELECT session_user AS user_name"
    set user_dollar_idx [string first \$ $user_name] 
    if {$user_dollar_idx != -1} {
        incr user_dollar_idx
        set user_name "[string range $user_name $user_dollar_idx end]" 
    }

    if {$TG_op == "UPDATE" || $TG_op == "DELETE"} {

      spi_exec -array C "select attname from pg_attribute where attrelid = $TG_relid and attnum > 0 and not attisdropped" {
          if {$C(attname) == "lastupdate"} {
              continue;
          }

          if {[ info exists NEW([ quote $C(attname) ]) ]} {
              if {[ info exists OLD([ quote $C(attname) ]) ]} {
                  if {$NEW([ quote $C(attname) ]) != $OLD([ quote $C(attname) ])} {
                      spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                          VALUES ('[ quote $log_time ]',
                              '[ quote $user_name ]',
                              '[ quote $TG_table_name ]',
                              '[ quote $C(attname) ]',
                              '[ quote $pkval ]',
                              '[ quote $NEW([ quote $C(attname) ]) ]',
                              '[ quote $OLD([ quote $C(attname) ]) ]',
                              '[ quote $TG_op ]',
                              $tx_name)"
                  }
              } else {
                  spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                      VALUES ('[ quote $log_time ]',
                          '[ quote $user_name ]',
                          '[ quote $TG_table_name ]',
                          '[ quote $C(attname) ]',
                          '[ quote $pkval ]',
                          '[ quote $NEW([ quote $C(attname) ]) ]',
                          null,
                          '[ quote $TG_op ]',
                          $tx_name)"
              }
          } else {
              if {[ info exists OLD([ quote $C(attname) ]) ]} {
                  spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                      VALUES ('[ quote $log_time ]',
                          '[ quote $user_name ]',
                          '[ quote $TG_table_name ]',
                          '[ quote $C(attname) ]',
                          '[ quote $pkval ]',
                          null,
                          '[ quote $OLD([ quote $C(attname) ]) ]',
                          '[ quote $TG_op ]',
                          $tx_name)"
              } else {
                  # no change -> do nothing
              }
          }
      }

    } else {
        if {$TG_op == "INSERT"} {
              spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                  VALUES ('[ quote $log_time ]',
                      '[ quote $user_name ]',
                      '[ quote $TG_table_name ]',
                      null,
                      '[ quote $pkval ]',
                      null,
                      null,
                      '[ quote $TG_op ]',
                      $tx_name)"
        }
    }


    if {$TG_op == "DELETE"} {
        return [array get OLD]
    } else {
        return [array get NEW]
    }
$$ LANGUAGE pltcl VOLATILE SECURITY DEFINER;


CREATE OR REPLACE FUNCTION logchanges() RETURNS trigger AS $$
    if {$TG_table_name == "wextid"} {
        if {$TG_op == "DELETE"} {
            return [array get OLD]
        } else {
            return [array get NEW]
        }
    }

    spi_exec "SELECT pg_attribute.attname AS pk
        FROM pg_index, pg_class, pg_attribute
        WHERE
          pg_class.oid = '[ quote $TG_table_name ]'::regclass AND
          indrelid = pg_class.oid AND
          pg_attribute.attrelid = pg_class.oid AND
          pg_attribute.attnum = any(pg_index.indkey)
          AND indisprimary"

    set modul null
    if {$TG_op == "INSERT" || $TG_op == "UPDATE"} {
        set pkval [ quote $NEW([ quote $pk ]) ]
        if {[info exists NEW(modul)]} {
            set modul '[ quote $NEW(modul) ]'
        }
    } else {
        set pkval [ quote $OLD([ quote $pk ]) ]
        if {[info exists OLD(modul)]} {
            set modul '[ quote $OLD(modul) ]'
        }
    }

    spi_exec "SELECT session_user AS user_name"
    set user_dollar_idx [string first \$ $user_name] 
    if {$user_dollar_idx != -1} {
        incr user_dollar_idx
        set user_name "[string range $user_name $user_dollar_idx end]" 
    }

    set operation "[string range $TG_op 0 0]"
    if {$TG_op == "INSERT"} {
        spi_exec "INSERT INTO changelog(tabulka, modul, id, operace, uzivatel)
            VALUES ('[ quote $TG_table_name ]',
                $modul,
                '[ quote $pkval ]',
                '[ quote $operation ]',
                '[ quote $user_name ]')"
    } elseif {$TG_op == "UPDATE" || $TG_op == "DELETE"} {
        set extid null

        if {[info exists OLD(kod)]} {
            append all_extids 'code:[ quote $OLD(kod) ]'
        }

        spi_exec "select attname AS fk from pg_attribute where attrelid = 'wextid'::regclass and attnum in (select conkey\[1\] from pg_constraint where conrelid = 'wextid'::regclass and confrelid = '[ quote $TG_table_name ]'::regclass)" {
            spi_exec "select extid from wextid where $fk = $pkval" {
                if {[info exists all_extids] && [string length $all_extids] > 0} {
                  append all_extids ","
                }
                append all_extids '[ quote $extid ]'
            }
        }

        if {[info exists all_extids] && [string length $all_extids] > 0} {
            set extid "array\[$all_extids\]"
        }

        spi_exec "INSERT INTO changelog(tabulka, modul, id, operace, uzivatel, extids)
            VALUES ('[ quote $TG_table_name ]',
                $modul,
                '[ quote $pkval ]',
                '[ quote $operation ]',
                '[ quote $user_name ]',
                $extid)"
    }

    if {$TG_op == "DELETE"} {
        return [array get OLD]
    } else {
        return [array get NEW]
    }
$$ LANGUAGE pltcl VOLATILE SECURITY DEFINER;


-- zvyseni verze databaze
INSERT INTO dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
    VALUES (nextval('seq_dbversion'), 717, 't', %appVersion%, clock_timestamp());

