-- Bug 41144 - [Legis] Přidat všechny sazby DPH z celé EU - Mini One Stop Shop



-- vytvori uuid sazby z kodu statu a jeji vyse
CREATE OR REPLACE FUNCTION uuidSazbyDph(
  varchar,
  usazbydph.szbdph%TYPE
)
RETURNS varchar AS
$BODY$
DECLARE
  a_stat ALIAS FOR $1;
  a_szbdph ALIAS FOR $2; 
  
  _uuid varchar;
BEGIN

  _uuid = to_char(a_szbdph, '09D09'); -- napr. cislo 7.50 -> retezec '07.50'
  _uuid = regexp_replace(_uuid, '[^0123456789]', '', 'g'); -- napr. retezec '07.50' -> '0750'
  _uuid = a_stat || '_' || _uuid || '_2015'; -- napr. retezec '07.50' -> DE_0750_2015
    
  return _uuid;
END;
$BODY$
  LANGUAGE 'plpgsql';

ALTER FUNCTION uuidsazbydph(character varying, numeric) OWNER TO %firmaRole%;



-- doplni jednu sazbu pro stat, pokud je to potreba
CREATE OR REPLACE FUNCTION doplnJednuSazbu(
  stat varchar, -- kod statu
  usazbydph.szbdph%TYPE, -- sazba
  usazbydph.typSzbDphK%TYPE -- typ sazby
  )
RETURNS void AS
$BODY$
DECLARE
  a_stat ALIAS FOR $1;
  a_szbdph ALIAS FOR $2; 
  a_typSzbDphK ALIAS FOR $3;

  _shodna integer;  
  _predchozi integer;  
  _jina integer;  

  _idstatu integer; 
  _uuid varchar;
BEGIN

--
-- Pokud budes pridavat nekdy dalsi sazby, pozor na to, ze zde by bylo lepsi 
-- hledat stat primo podle jeho kodu a ne uuid, viz bug 41268 comment#2.
--

  select idstatu into _idstatu from astaty where kod = a_stat;
  
  if (_idstatu is null) then
    raise notice 'Nepridavam pro stat % dph % %, ciselnik statu neobsahuje stat s UUID = "CZ_%" ani "SK_%"!.', a_stat, a_szbdph, a_typSzbDphK, a_stat, a_stat;
    return;
  end if;
    

  -- Neexistuje uz v db nastavena shodna dph, ktera je platna i v roce 2015? Pak je zbytecne ji pridavat.
  
  _shodna = 0;
  
  select idsazbydph into _shodna from usazbydph
    where typSzbDphK = a_typSzbDphK and szbdph = a_szbdph and idstatu = _idstatu 
    and platioddata <= '2015-01-01' and (platidodata is null or platidodata > '2015-01-01')
    order by platioddata desc limit 1; 

  if(_shodna > 0) then
    raise notice 'Nepridavam pro stat % dph % %, uz existuje (id %).', a_stat, a_szbdph, a_typSzbDphK, _shodna;
    return;
  end if;


  -- Neexistuje uz v db nastavena jina dph, ktera je platna od 1. 1. 2015? Pak ji nechceme menit.
-- tady je malá odlišnost od upgrade skriptu z bug 41144 -
-- bez ohledu na počátek platnosti ukončíme ty sazby dph, které jsou aktuálně platné

  -- Neexistuje uz v db nastavena jina dph, ktera je platna po 1. 1. 2015? Pak ji musime ukoncit.

  _predchozi = 0;
  
  select idsazbydph into _predchozi from usazbydph
    where typSzbDphK = a_typSzbDphK and szbdph != a_szbdph and idstatu = _idstatu 
    and (platidodata is null or platidodata > '2015-01-01')
    order by platioddata desc limit 1; 

  if(_predchozi > 0) then
    -- sazbu ukončíme k 1.1.2015, protože může existovat i ta z bug 41144
    -- teoreticky může existovat sazeb, které splňují podmínku více - zde použiji celou podmínku where z dotazu výše
    update usazbydph set platidodata = '2014-12-31'  where typSzbDphK = a_typSzbDphK and szbdph != a_szbdph and idstatu = _idstatu 
        and (platidodata is null or platidodata > '2015-01-01');
    -- abychom neměli sazbu, která skončila dřív než začala a nemuseli jsme ji mazat - nastavím platnost od na datum platnosti do
    update usazbydph set platioddata = '2014-12-31'  where typSzbDphK = a_typSzbDphK and szbdph != a_szbdph and idstatu = _idstatu 
        and (platidodata is null or platidodata > '2015-01-01') and platioddata = '2015-01-01';
    raise notice 'Ukoncuji platnost predchozi sazby % (id %) pro stat %.', a_typSzbDphK, _predchozi, a_stat;
  end if;

  -- ulozeni dph platne od 1. 1. 2015
  
  _uuid = uuidSazbyDph(a_stat, a_szbdph); 
    
  INSERT INTO usazbydph (lastupdate, platidodata, platioddata, popis, poznam, szbdph, typszbdphk, idstatu, uuid)
      VALUES ('2015-01-01 12:00:00.000', NULL, '2015-01-01', NULL, NULL, a_szbdph, a_typSzbDphK, _idstatu, _uuid);

  raise notice 'Pridavam pro stat % dph % %.', a_stat, a_szbdph, a_typSzbDphK;
  
END;
$BODY$
  LANGUAGE 'plpgsql';

ALTER FUNCTION doplnjednusazbu(character varying, numeric, character varying) OWNER TO %firmaRole%;



-- doplni postupne jednotlive sazby pro stat, pokud je to potreba
CREATE OR REPLACE FUNCTION doplnSazby(
  stat varchar, -- kod statu
  usazbydph.szbdph%TYPE,  -- zakl sazba
  usazbydph.szbdph%TYPE,  -- sniz sazba
  usazbydph.szbdph%TYPE   -- sniz2 sazba
  )
RETURNS void AS
$BODY$
DECLARE
  a_stat ALIAS FOR $1;
  a_szbdphZakl ALIAS FOR $2; 
  a_szbdphSniz ALIAS FOR $3; 
  a_szbdphSniz2 ALIAS FOR $4; 
  
  _typSzbDphK varchar;
  _szbdph usazbydph.szbdph%TYPE; 
BEGIN

  -- doplneni typSzbDph.dphZakl

  _szbdph = a_szbdphZakl;
  _typSzbDphK = 'typSzbDph.dphZakl';
  
  PERFORM doplnJednuSazbu(a_stat, _szbdph, _typSzbDphK);

  -- doplneni typSzbDph.dphSniz

  _szbdph = a_szbdphSniz;
  _typSzbDphK = 'typSzbDph.dphSniz';
  
  PERFORM doplnJednuSazbu(a_stat, _szbdph, _typSzbDphK);
  
  -- doplneni typSzbDph.dphSniz2

  _szbdph = a_szbdphSniz2;
  _typSzbDphK = 'typSzbDph.dphSniz2';
  
  PERFORM doplnJednuSazbu(a_stat, _szbdph, _typSzbDphK);  

END;
$BODY$
  LANGUAGE 'plpgsql';

ALTER FUNCTION doplnsazby(character varying, numeric, numeric, numeric) OWNER TO %firmaRole%;

-- vlozeni sazeb pro rok 2015 pro Lucembursko

select doplnSazby('LU', 17.00, 8.00,  3.00);

-- odliseni sniz2 sazeb
update uSazbyDph set uuid = uuid || '_2' where typszbdphk = 'typSzbDph.dphSniz2' and uuid !~ '_2$';

-- uklid
DROP FUNCTION uuidsazbydph(character varying, numeric);
DROP FUNCTION doplnjednusazbu(character varying, numeric, character varying);
DROP FUNCTION doplnsazby(character varying, numeric, numeric, numeric);

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
   values (nextval('seq_dbversion'), 908, 't', %appVersion%, clock_timestamp());
