--------------------------------------------------------------------------------------------------------------------------------------------------
-- Bug 43673 - "Nekorektně" vyplněný doklad se zbytkem může při změna účtů na navázeaném bankovním dokladu vyvolat chybu konkurentního přístupu 
--------------------------------------------------------------------------------------------------------------------------------------------------


--
-- TOC entry 27 (class 1255 OID 238789)
-- Dependencies: 3 673
-- Name: ddoklintupdateucty(); Type: FUNCTION; Schema: public; Owner: %firmaRole%
--

CREATE OR REPLACE FUNCTION ddoklintupdateucty() RETURNS trigger
    AS $$
declare idPoloz int;
        idDokl int;
        kopirovatKR boolean;
        ucMd int;
        ucDal int;
        ucNastav int;
        _zdrojprofak boolean;
        _hromfakt boolean;
begin
  kopirovatKR = false;
  -- pokud se zmenil nejaky ucet a mame vyplneny typ zauctovani zkusime nejprve rozhodnout jaky ucet prenaset podle nej
  if (compareId(NEW.idPrimUcet, OLD.idPrimUcet) = false or compareId(NEW.idProtiUcet, OLD.idProtiUcet) = false and NEW.idtypucop is not null) then
    -- MD - prim ucet, DAL - proti ucet
    select P.idprotiucetp, P.idprotiucetv into ucDal, ucMd from utypucop as P where NEW.idtypucop = P.idtypucop;
    if (ucMd is null and ucDal is not null and not compareId(NEW.idPrimUcet, OLD.idPrimUcet) ) then        -- na predpisu zauctovani je zadan jen ucet Dal, tj prenasime ucet Md
      kopirovatKR = true;
      ucNastav = NEW.idPrimUcet;
    end if;
    if (ucDal is null and ucMd is not null and not compareId(NEW.idProtiUcet, OLD.idProtiUcet) ) then        -- na predpisu zauctovani je zadan jen ucet Md, tj prenasime ucet Dal
      kopirovatKR = true;
      ucNastav = NEW.idProtiUcet;
    end if;
    
    if kopirovatKR then
      for idDokl in (select idDoklFak from dDoklFak where compareId(idPrimUcet, ucNastav) = false and idDoklFak in
          (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK in ('typVazbyDokl.uhradaKR','typVazbyDokl.preceneni','typVazbyDokl.uhradaZB')))
      loop
        execute dNastavUcetDokladu(0, idDokl, 0, ucNastav);
      end loop;
    end if;
  end if;

  --pokud dojde ke zmene protiuctu
  if (compareId(NEW.idProtiUcet, OLD.idProtiUcet) = false) then
    if (NEW.modul = 'POK' and NEW.typPohybuK = 'typPohybu.vydej') then             -- pokladni vydej
      -- vazba na sklad
      for idDokl in (select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.hlavaSkl' and idTabA = NEW.idDoklInt and modulA = NEW.modul and
            compareId((select idProtiUcet from dDoklSklad where idDoklSklad = idTabB), NEW.idProtiUcet) = false)
      loop
        select zdrojprofak, hromfakt into _zdrojprofak, _hromfakt from ddoklsklad where iddoklsklad = iddokl;
        if _zdrojprofak = false or _hromfakt = false then     -- pokud byl skladovy doklad pridany hromadnou fakturaci tak na nej neprenasime
          execute dNastavUcetDokladu(2, idDokl, 1, NEW.idProtiUcet);
        end if;
      end loop;
    end if;

    -- uprava faktur vazbami uhrada a vazbaZDD, pouze pokud ucet nesouhlasi aby nedochazelo k zacykleni
    for idDokl in (select idDoklFak from dDoklFak where compareId(idPrimUcet, NEW.idProtiUcet) = false and idDoklFak in
        (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK in ('typVazbyDokl.uhrada','typVazbyDokl.vazbaZDD')))
    loop
        execute dNastavUcetDokladu(0, idDokl, 0, NEW.idProtiUcet);
    end loop;

    -- odpocet zalohy - polozka dokladu
    for idPoloz in (select idTabA from dVazebTab where typVazbyK = 'typVazbyDokl.odpocetZALOHY' and idTabB in
            (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK = 'typVazbyDokl.uhradaZALOHY')) loop
      if (NEW.typPohybuK = 'typPohybu.prijem') then
        update dPolFak set lastupdate = now(), idZklDalUcet = NEW.idProtiUcet where idPolFak = idPoloz and compareId(idZklDalUcet, NEW.idProtiUcet) = false and compareId(idZklDalUcet, OLD.idProtiUcet) = true;
      else
        update dPolFak set lastupdate = now(), idZklMdUcet  = NEW.idProtiUcet where idPolFak = idPoloz and compareId(idZklMdUcet,  NEW.idProtiUcet) = false and compareId(idZklMdUcet,  OLD.idProtiUcet) = true;
      end if;
    end loop;

    -- uhradaKR, preceneni
    if (not kopirovatKR and not compareId(OLD.idPrimUcet, OLD.idProtiUcet)) then  -- nekopirovali jsme a ucty na internim dokladu nebyli stejne
      for idDokl in (select idDoklFak from dDoklFak where compareId(idPrimUcet, NEW.idProtiUcet) = false and compareId(idPrimUcet, OLD.idProtiUcet) = true and idDoklFak in
          (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK in ('typVazbyDokl.uhradaKR','typVazbyDokl.preceneni','typVazbyDokl.uhradaZB')))
      loop
        execute dNastavUcetDokladu(0, idDokl, 0, NEW.idProtiUcet);
      end loop;
    end if;
  end if;

  if (compareId(NEW.idPrimUcet, OLD.idPrimUcet) = false) then
    -- uhradaKR, preceneni
    if (not kopirovatKR and not compareId(OLD.idPrimUcet, OLD.idProtiUcet)) then  -- nekopirovali jsme a ucty na internim dokladu nebyli stejne
      for idDokl in (select idDoklFak from dDoklFak where compareId(idPrimUcet, NEW.idPrimUcet) = false and compareId(idPrimUcet, OLD.idPrimUcet) = true and idDoklFak in
          (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK in ('typVazbyDokl.uhradaKR','typVazbyDokl.preceneni','typVazbyDokl.uhradaZB')))
      loop
        execute dNastavUcetDokladu(0, idDokl, 0, NEW.idPrimUcet);
      end loop;
    end if;
  end if;

  return NEW;
end;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION public.ddoklintupdateucty() OWNER TO %firmaRole%;



-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 946, 't', %appVersion%, clock_timestamp());
