--------------------------------------------------------------------------------------------------
-- Bug 44417 - Není možné vyfakturovat objednávku
--------------------------------------------------------------------------------------------------

-- i_novyDoklad = nova objednavka
CREATE OR REPLACE FUNCTION dgetstavnaskladeaktualni(i_idcenik integer, i_idsklad integer, i_datum date,
 i_idfirma integer, i_iddoklobch integer, i_novyDoklad boolean)
  RETURNS numeric AS $$
declare
begin
    RETURN dgetstavnaskladeaktualni(i_idcenik, i_idsklad, i_datum, i_idfirma, array[i_iddoklobch], i_novyDoklad);  
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dgetstavnaskladeaktualni(integer, integer, date, integer, integer,boolean) OWNER TO %firmaRole%;

/* i_iddoklobch - pole, pokud hledáme pro více obchodních dokladů. Pokud hledáme jen pro jeden obch. doklad, bude délky 1.
 *     Pokud nehledáme pro žádný obchodní doklad, bude délky 1 s jediným elementem "-1".
 * */
CREATE OR REPLACE FUNCTION dgetstavnaskladeaktualni(i_idcenik integer, i_idsklad integer, i_datum date,
 i_idfirma integer, i_iddoklobch integer[], i_novyDoklad boolean)
  RETURNS numeric AS $$
declare
  _stavMj numeric(19,6); 
  _firmaRezerMj numeric(19,6); 
  _celkemRezerMj numeric(19,6); 
begin
  SELECT dgetstavnaskladeaktualni(i_idcenik, i_idsklad, i_datum) INTO _stavMj; 
  
  -- zjistime kolik je celkem rezervovano - od/do ignorujeme (bug 25844, 25789), muselo by se zohlednovat uz pri plneni pole skladmj a nebo toto pole idealne vyhodit
  select coalesce(sum(skladmj), 0) into _celkemRezerMj from srezervace where idcenik = i_idcenik and idsklad = i_idsklad;
   
  IF (i_idfirma > 0) THEN 
    -- array_upper - zjištění délky pole
    -- podmínka - viz možný obsah i_iddoklobch v hlavičce metody
    IF ((array_upper(i_iddoklobch,1) = 1 and i_iddoklobch[0] > 0) or array_upper(i_iddoklobch,1) > 1) THEN 
        -- zjistime kolik ma firma rezervovano  
        -- fce any - pro hledání "in array"
        select coalesce(sum(r.skladmj), 0) into _firmaRezerMj from srezervace r 
            left join dpolobch po on r.idpolobch = po.idpolobch
            where r.idfirmy = i_idfirma and r.idcenik = i_idcenik and r.idsklad = i_idsklad and (po.iddoklobch is null or po.iddoklobch  = any(i_iddoklobch)); 
    ELSIF (i_novyDoklad) THEN
            -- jedná se o novou objednávku - zjistime kolik ma firma rezervovano, ale v tomto pripade jen rezervace bez dokladu
        select coalesce(sum(r.skladmj), 0) into _firmaRezerMj from srezervace r  
            left join dpolobch po on r.idpolobch = po.idpolobch 
            where r.idfirmy = i_idfirma and r.idcenik = i_idcenik and r.idsklad = i_idsklad and po.iddoklobch is null;  
    ELSE
        select coalesce(sum(skladmj), 0) into _firmaRezerMj from srezervace where idfirmy = i_idfirma and idcenik = i_idcenik and idsklad = i_idsklad; 
    END IF;
  ELSE 
    _firmaRezerMj := 0; 
  END IF; 
   
  -- spocteme kolik mu muzeme dat 
  _stavMj := minValue(_stavMj, maxValue(_stavMj - _celkemRezerMj, 0) + _firmaRezerMj); 
   
  RETURN _stavMj; 
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dgetstavnaskladeaktualni(integer, integer, date, integer, integer[],boolean) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 971, 't', %appVersion%, clock_timestamp());